
from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkIpNeighbor(Base):
    '''
    Operations that can be performed on arp tables
    '''
    moid = 'ha-cli-handler-network-ip-neighbor'
    def list(self, version=None):
        '''
        List ARP table entries
        :param version: string, IP version :  [4, 6, all]
        :returns: vim.EsxCLI.network.ip.neighbor.list.NetworkNeighbor[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.neighbor.List',
                            version=version,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VmfsDatastoreExtendSpec(vim, *args, **kwargs):
    '''Specification to increase the capacity of a VMFS datastore by adding one or
    more new extents to the datastore. All the extents to be added must be on the
    same disk. Extension is different from creation in that the VMFS creation
    specification need not be specified.'''

    obj = vim.client.factory.create('{urn:vim25}VmfsDatastoreExtendSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'extent', 'partition', 'diskUuid' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj



from json import loads, dumps
from logging import getLogger
from infi.pyutils.lazy import cached_method, clear_cache
from pyvisdk import __version__

logger = getLogger(__name__)

from .errors import ExtensionAlreadyRegisteredException, ExtensionNotRegisteredException
from .resources import ExtensionResourceFactory

class ExtensionFacade(object):
    DESCRIPTION = "pyvisdk"
    VERSION = __version__

    def __init__(self, client, key):
        super(ExtensionFacade, self).__init__()
        self._client = client
        self._managed_object = self._client.service_content.extensionManager
        self._resources_dict = ExtensionResourceFactory.get_dict(self._client, key)
        self._key = key

    # Extension Registration

    @cached_method
    def is_registered(self):
        return len(self._get_extensions_data_objects()) > 0

    def register(self):
        if self.is_registered():
            raise ExtensionAlreadyRegisteredException()
        extension = self._new_extension()
        self._managed_object.RegisterExtension(extension)
        clear_cache(self)

    # Tasks management

    def is_task_registered(self, task_name):
        return task_name in self._get_tasks().values()

    def get_task_id(self, task_name):
        if not self.is_registered():
            raise ExtensionNotRegisteredException()
        if not self.is_task_registered(task_name):
            raise KeyError(task_name)
        tasks = self._get_tasks()
        reverted = {value:key for key, value in tasks.items()}
        return reverted[task_name]

    def register_task(self, task_name):
        task_id = self._generate_id()
        self._add_id(task_id, task_name)
        self._add_task(task_id, task_name)

    # Utility methods

    def _generate_id(self):
        from uuid import uuid4
        id = str(uuid4())
        return id if id not in self._get_ids() else self._generate_id()

    @cached_method
    def _get_extensions_data_objects(self):
        return [extension for extension in self._managed_object.extensionList
                if extension.key == self._key]

    def _get_ids(self):
        return self._resources_dict

    def _get_tasks(self):
        ids = self._get_ids()
        return {task.taskID: ids[task.taskID]
                for task in self._get_extensions_data_objects()[0].taskList
                if ids.get(task.taskID)}

    def _add_task(self, task_id, task_name):
        extension = self._get_extensions_data_objects()[0]
        tasks = self._get_tasks()
        tasks[task_id] = task_name
        extension.taskList = [self._new_task_type_info(task_id) for task_id in tasks.keys()]
        task_resources = [("{}.label".format(key), value) for key, value in tasks.items()]
        # TODO events
        extension.resourceList = [self._new_task_extension_resource_info('en', 'task', task_resources)]
        self._managed_object.UpdateExtension(extension)
        clear_cache(self)

    def _add_id(self, key, value):
        self._resources_dict[key] = value

    def _new_extension(self):
        from pyvisdk.do.extension import Extension
        from pyvisdk.do.description import Description
        description = Description(self._client, label=self.DESCRIPTION, summary='')
        return Extension(self._client, description=description, key=self._key,
                         lastHeartbeatTime=self._get_heartbeat(),
                         version=self.VERSION)

    def _get_heartbeat(self):
        return self._client.service_instance.CurrentTime()

    def _new_task_type_info(self, task_id):
        from pyvisdk.do.extension_task_type_info import ExtensionTaskTypeInfo
        return ExtensionTaskTypeInfo(self._client, task_id)

    def _new_task_extension_resource_info(self, locale, module, items):
        from pyvisdk.do.extension_resource_info import ExtensionResourceInfo
        from pyvisdk.do.key_value import KeyValue
        data = [KeyValue(self._client, key, value) for key, value in items]
        return ExtensionResourceInfo(self._client, locale=locale, module=module, data=data)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineRelocateDiskMoveOptions = Enum(
    'createNewChildDiskBacking',
    'moveAllDiskBackingsAndAllowSharing',
    'moveAllDiskBackingsAndDisallowSharing',
    'moveChildMostDiskBacking',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostPortGroupProfile(vim, *args, **kwargs):
    '''The HostPortGroupProfile data object represents the subprofile for a port group
    that will be used by the ESX host. If a profile plug-in defines policies or
    subprofiles, use the policy or property list to access the additional
    configuration data.'''

    obj = vim.client.factory.create('{urn:vim25}HostPortGroupProfile')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'ipConfig', 'key', 'name', 'networkPolicy', 'vlan', 'vswitch', 'enabled' ]
    optional = [ 'policy', 'profileTypeName', 'profileVersion', 'property', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def PlatformConfigFault(vim, *args, **kwargs):
    '''A PlatformConfigFault is a catch-all fault indicating that some error has
    occurred regarding the configuration of the host. Data about the fault is
    available and will be presented as a platform specific string.This information
    carried by this fault cannot be localized. Most likely this information will
    already have been localized to the locale of the server that generated this
    fault. Where possible, a more specific fault will be thrown.'''

    obj = vim.client.factory.create('{urn:vim25}PlatformConfigFault')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'text', 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def TaskInfo(vim, *args, **kwargs):
    '''This data object type contains all information about a task. A task represents
    an operation performed by VirtualCenter or ESX.'''

    obj = vim.client.factory.create('{urn:vim25}TaskInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 9:
        raise IndexError('Expected at least 10 arguments got: %d' % len(args))

    required = [ 'cancelable', 'cancelled', 'descriptionId', 'eventChainId', 'key', 'queueTime',
        'reason', 'state', 'task' ]
    optional = [ 'changeTag', 'completeTime', 'description', 'entity', 'entityName', 'error',
        'locked', 'name', 'parentTaskKey', 'progress', 'result', 'rootTaskKey',
        'startTime', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostPatchManager(BaseEntity):
    '''This managed object is the interface for scanning and patching an ESX server.
    VMware publishes updates through its external website. A patch update is
    synonymous with a bulletin. An update may contain many individual patch
    binaries, but its installation and uninstallation are atomic.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostPatchManager):
        super(HostPatchManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def CheckHostPatch_Task(self, metaUrls=None, bundleUrls=None, spec=None):
        '''Check the list of metadata and returns the dependency, obsolete and conflict
        information The operation is cancelable through the returned Task object. No
        integrity checks are performed on the metadata.
        
        :param metaUrls: a list of urls pointing to metadata.zip.
        
        :param bundleUrls: a list of urls pointing to an "offline" bundle.
        
        :param spec: 
        
        '''
        return self.delegate("CheckHostPatch_Task")(metaUrls, bundleUrls, spec)
    
    def InstallHostPatch_Task(self, repository, updateID, force=None):
        '''<b>Deprecated.</b> <i>Method is deprecated, use InstallHostPatchV2_Task
        instead.</i> Patch the host. The operation is not cancelable. If the patch
        installation failed, an atomic rollback of the installation will be attempted.
        Manual rollback is required if the atomic rollback failed, see
        PatchInstallFailed for details.
        
        :param repository: Location of the repository that contains the bulletin depot. The depot must be organized as a flat collection of bulletins with each one being a folder named after the bulletin ID. Each folder must contain both update metadata and required binaries.
        
        :param updateID: The update to be installed on the host.
        
        :param force: Specify whether to force reinstall an update. By default, installing an already-installed update would fail with the PatchAlreadyInstalled fault. If force is set to true, the update will be forcifully reinstalled, thus overwriting the already installed update.
        
        '''
        return self.delegate("InstallHostPatch_Task")(repository, updateID, force)
    
    def InstallHostPatchV2_Task(self, metaUrls=None, bundleUrls=None, vibUrls=None, spec=None):
        '''Patch the host. The operation is not cancelable. If the patch installation
        failed, an atomic rollback of the installation will be attempted. Manual
        rollback is required if the atomic rollback failed, see PatchInstallFailed for
        details.
        
        :param metaUrls: A list of urls pointing to metadata.zip.
        
        :param bundleUrls: a list of urls pointing to an "offline" bundle.
        
        :param vibUrls: The urls of update binary files to be installed.
        
        :param spec: 
        
        '''
        return self.delegate("InstallHostPatchV2_Task")(metaUrls, bundleUrls, vibUrls, spec)
    
    def QueryHostPatch_Task(self, spec=None):
        '''Query the host for installed bulletins.
        
        :param spec: 
        
        '''
        return self.delegate("QueryHostPatch_Task")(spec)
    
    def ScanHostPatch_Task(self, repository, updateID=None):
        '''<b>Deprecated.</b> <i>As of VI API 4.0, use ScanHostPatchV2_Task.</i> Scan the
        host for the patch status. The operation is cancelable through the returned
        Task object. Integrity checks are performed on the metadata only during the
        scan operation.
        
        :param repository: Location of the repository that contains the bulletin depot. The depot must be organized as a flat collection of bulletins with each one being a folder named after the bulletin ID. Each folder must contain the full update metadata.
        
        :param updateID: The updates to scan. Wildcards can be used to specify the update IDs. The wildcards will be expanded to include all updates whose IDs match the specified wildcard and whose metadata is available in the repository. Specifying no update is equivalent to a wildcard "*". In this case all updates available in the repository will be scanned.
        
        '''
        return self.delegate("ScanHostPatch_Task")(repository, updateID)
    
    def ScanHostPatchV2_Task(self, metaUrls=None, bundleUrls=None, spec=None):
        '''Scan the host for the patch status. The operation is cancelable through the
        returned Task object. Integrity checks are performed on the metadata only
        during the scan operation.
        
        :param metaUrls: a list of urls pointing to metadata.zip.
        
        :param bundleUrls: a list of urls pointing to an "offline" bundle.
        
        :param spec: 
        
        '''
        return self.delegate("ScanHostPatchV2_Task")(metaUrls, bundleUrls, spec)
    
    def StageHostPatch_Task(self, metaUrls=None, bundleUrls=None, vibUrls=None, spec=None):
        '''Stage the vib files to esx local location and possibly do some run time check.
        
        :param metaUrls: A list of urls pointing to metadata.zip.
        
        :param bundleUrls: a list of urls pointing to an "offline" bundle.
        
        :param vibUrls: The urls of update binary files to be staged.
        
        :param spec: 
        
        '''
        return self.delegate("StageHostPatch_Task")(metaUrls, bundleUrls, vibUrls, spec)
    
    def UninstallHostPatch_Task(self, bulletinIds=None, spec=None):
        '''Uninstall patch from the host. The operation is not cancelable.
        
        :param bulletinIds: A list of bulletin IDs to be removed.
        
        :param spec: 
        
        '''
        return self.delegate("UninstallHostPatch_Task")(bulletinIds, spec)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualKeyboardOption(vim, *args, **kwargs):
    '''The VirtualKeyboardOption data object type contains the options for the virtual
    keyboard class.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualKeyboardOption')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'deprecated', 'hotRemoveSupported', 'plugAndPlay', 'type' ]
    optional = [ 'autoAssignController', 'backingOption', 'connectOption', 'controllerType',
        'defaultBackingOptionIndex', 'licensingLimit', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiNetworkportal(Base):
    '''
    Operations that can be performed on iSCSI Network Portal (iSCSI vmknic)
    '''
    moid = 'ha-cli-handler-iscsi-networkportal'
    def add(self, adapter, nic, force=None):
        '''
        Add a network portal for iSCSI adapter
        :param adapter: string, The iSCSI adapter name.
        :param force: boolean, The force flag (force add of iSCSI vmknic even it's not compatible for iscsi multipathing)
        :param nic: string, The iSCSI network portal (bound vmknic)
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.networkportal.Add',
                            adapter=adapter,
                            force=force,
                            nic=nic,
                            )
    def list(self, adapter=None):
        '''
        List Network Portal for iSCSI Adapter
        :param adapter: string, The iSCSI adapter name.
        :returns: vim.EsxCLI.iscsi.networkportal.list.NetworkPortal[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.networkportal.List',
                            adapter=adapter,
                            )
    def remove(self, adapter, nic, force=None):
        '''
        Remove a network portal for iSCSI adapter
        :param adapter: string, The iSCSI adapter name.
        :param force: boolean, The force flag (force removal of iSCSI vmknic when sessions are active using it)
        :param nic: string, The iSCSI network portal (bound vmknic)
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.networkportal.Remove',
                            adapter=adapter,
                            force=force,
                            nic=nic,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemModule(Base):
    '''
    Operations that allow manipulation of the VMkernel loadable modules and device drivers. Operations include load, list and setting options.
    '''
    moid = 'ha-cli-handler-system-module'
    def load(self, module, force=None):
        '''
        Load a VMkernel module with the given name if it is enabled. If the module is disabled then the use of --force is required to load the module.
        :param force: boolean, Ignore the enabled/disabled state of this module and force it to load.
        :param module: string, The name of the VMkernel module to load.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.module.Load',
                            force=force,
                            module=module,
                            )
    def set(self, enabled, module, force=None):
        '''
        Allow enabling and disabling of a VMkernel module.
        :param enabled: boolean, Set to true to enable the module, set to false to disable the module.
        :param force: boolean, Skip VMkernel module validity checks and set options for a module (or alias) with the given name.
        :param module: string, The name of the VMkernel module to set options for.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.module.Set',
                            enabled=enabled,
                            force=force,
                            module=module,
                            )
    def list(self, enabled=None, loaded=None):
        '''
        List the VMkernel modules that the system knows about.
        :param enabled: boolean, List the enabled / disabled VMkernel modules and device drivers.
        :param loaded: boolean, List the loaded / not loaded VMkernel modules and device drivers.
        :returns: vim.EsxCLI.system.module.list.Module[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.module.List',
                            enabled=enabled,
                            loaded=loaded,
                            )
    def get(self, module):
        '''
        Show the ELF header information for the given VMkernel module.
        :param module: string, The name of the VMkernel module to get the option string for.
        :returns: vim.EsxCLI.system.module.get.ModuleDetails
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.module.Get',
                            module=module,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostPatchManagerIntegrityStatus = Enum(
    'digestMismatch',
    'keyExpired',
    'keyNotFound',
    'keyRevoked',
    'notEnoughSignatures',
    'validated',
    'validationError',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

PerfStatsType = Enum(
    'absolute',
    'delta',
    'rate',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualMachineRuntimeInfo(vim, *args, **kwargs):
    '''The RuntimeInfo data object type provides information about the execution state
    and history of a virtual machine.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualMachineRuntimeInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'connectionState', 'consolidationNeeded', 'faultToleranceState',
        'numMksConnections', 'powerState', 'recordReplayState', 'toolsInstallerMounted' ]
    optional = [ 'bootTime', 'cleanPowerOff', 'dasVmProtection', 'device', 'host',
        'maxCpuUsage', 'maxMemoryUsage', 'memoryOverhead', 'minRequiredEVCModeKey',
        'needSecondaryReason', 'question', 'suspendInterval', 'suspendTime',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualDeviceFileExtension = Enum(
    'dsk',
    'flp',
    'iso',
    'rdm',
    'vmdk',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualDiskMode = Enum(
    'append',
    'independent_nonpersistent',
    'independent_persistent',
    'nonpersistent',
    'persistent',
    'undoable',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkFenceNetwork(Base):
    '''
    Commands to list fence network information
    '''
    moid = 'ha-cli-handler-network-fence-network'
    def list(self, vdsname, fenceid=None):
        '''
        Get all fence network info on the vds.
        :param fenceid: long, The fence id used to retrieve fence info.
        :param vdsname: string, The vds name used to retrieve fence info.
        :returns: vim.EsxCLI.network.fence.network.list.FenceNetworkList
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.fence.network.List',
                            fenceid=fenceid,
                            vdsname=vdsname,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

TaskFilterSpecTimeOption = Enum(
    'completedTime',
    'queuedTime',
    'startedTime',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageVmfsSnapshotExtent(Base):
    '''
    Manage VMFS snapshot extents.
    '''
    moid = 'ha-cli-handler-storage-vmfs-snapshot-extent'
    def list(self, volumelabel=None, volumeuuid=None):
        '''
        List extents of unresolved snapshots/replicas of VMFS volume.
        :param volumelabel: string, The VMFS volume label of the target snapshot to enumerate.
        :param volumeuuid: string, The VMFS volume uuid of the target snapshot to enumerate.
        :returns: vim.EsxCLI.storage.vmfs.snapshot.extent.list.VMFSSnapshotExtent[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.vmfs.snapshot.extent.List',
                            volumelabel=volumelabel,
                            volumeuuid=volumeuuid,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def DatastoreFileEvent(vim, *args, **kwargs):
    '''Base class for events related to datastore file and directory
    operations.Property inherited from DatastoreEvent refers to the destination
    datastore in case there is more than datastore involved in the operation.'''

    obj = vim.client.factory.create('{urn:vim25}DatastoreFileEvent')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'targetFile', 'chainId', 'createdTime', 'key', 'userName' ]
    optional = [ 'datastore', 'changeTag', 'computeResource', 'datacenter', 'ds', 'dvs',
        'fullFormattedMessage', 'host', 'net', 'vm', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ComputeResourceHostSPBMLicenseInfoHostSPBMLicenseState = Enum(
    'licensed',
    'unknown',
    'unlicensed',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.history_collector import HistoryCollector

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class TaskHistoryCollector(HistoryCollector):
    '''TaskHistoryCollector provides a mechanism for retrieving historical data and
    updates when the server appends new tasks.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.TaskHistoryCollector):
        super(TaskHistoryCollector, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def latestPage(self):
        '''The items in the 'viewable latest page'. As new tasks that match the
        collector's TaskFilterSpec are created, they are added to this page, and the
        oldest tasks are removed from the collector to keep the size of the page to
        that allowed by SetCollectorPageSize.'''
        return self.update('latestPage')

    
    
    def ReadNextTasks(self, maxCount):
        '''Reads the 'scrollable view' from the current position. The scrollable position
        is moved to the next newer page after the read. No item is returned when the
        end of the collector is reached.
        
        :param maxCount: The maximum number of items in the page.
        
        '''
        return self.delegate("ReadNextTasks")(maxCount)
    
    def ReadPreviousTasks(self, maxCount):
        '''Reads the 'scrollable view' from the current position. The scrollable position
        is then moved to the next older page after the read. No item is returned when
        the head of the collector is reached.
        
        :param maxCount: The maximum number of items in the page.
        
        '''
        return self.delegate("ReadPreviousTasks")(maxCount)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class HardwareMemory(Base):
    '''
    Memory information.
    '''
    moid = 'ha-cli-handler-hardware-memory'
    def get(self):
        '''
        Get information about memory.
        :returns: vim.EsxCLI.hardware.memory.get.Memory
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.hardware.memory.Get',
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineFlagInfoVirtualExecUsage = Enum(
    'hvAuto',
    'hvOff',
    'hvOn',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

PerformanceManagerUnit = Enum(
    'joule',
    'kiloBytes',
    'kiloBytesPerSecond',
    'megaBytes',
    'megaBytesPerSecond',
    'megaHertz',
    'microsecond',
    'millisecond',
    'number',
    'percent',
    'second',
    'watt',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

CustomizationSysprepRebootOption = Enum(
    'noreboot',
    'reboot',
    'shutdown',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def TraversalSpec(vim, *args, **kwargs):
    '''The TraversalSpec data object type specifies how to derive a new set of objects
    to add to the filter.It specifies a property path whose value is either another
    managed object or an array of managed objects included in the set of objects
    for consideration. This data object can also be named, using the "name" field
    in the base type.'''

    obj = vim.client.factory.create('{urn:vim25}TraversalSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'path', 'type' ]
    optional = [ 'selectSet', 'skip', 'name', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ObjectSpec(vim, *args, **kwargs):
    '''Within a PropertyFilterSpec, the ObjectSpec data object type specifies the
    managed object at which the filter begins to collect the managed object
    references and properties specified by the associated PropertySpec set. If the
    "skip" property is present and set to true, then the filter does not check to
    see if the starting object's type matches any of the types listed in the
    associated sets of PropertySpec data objects.If the selectSet property is
    present, then this specifies additional objects to filter. These objects are
    defined by one or more SelectionSpec objects.'''

    obj = vim.client.factory.create('{urn:vim25}ObjectSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'obj' ]
    optional = [ 'selectSet', 'skip', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

CannotPowerOffVmInClusterOperation = Enum(
    'guestShutdown',
    'guestSuspend',
    'powerOff',
    'suspend',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class LicenseAssignmentManager(BaseEntity):
    ''''''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.LicenseAssignmentManager):
        super(LicenseAssignmentManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def QueryAssignedLicenses(self, entityId=None):
        '''Get information about all the licenses associated with an entity
        
        :param entityId: ID of the entity. E.g. HostSystem.
        
        '''
        return self.delegate("QueryAssignedLicenses")(entityId)
    
    def RemoveAssignedLicense(self, entityId):
        '''Remove licenses associated with an entity
        
        :param entityId: ID of the entity. E.g. HostSystem.
        
        '''
        return self.delegate("RemoveAssignedLicense")(entityId)
    
    def UpdateAssignedLicense(self, entity, licenseKey, entityDisplayName=None):
        '''Update the license associated with an entity
        
        :param entity: ID of the entity. E.g. HostSystem.
        
        :param licenseKey: A license.
        
        :param entityDisplayName: Display name for the entity
        
        '''
        return self.delegate("UpdateAssignedLicense")(entity, licenseKey, entityDisplayName)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VmfsDatastoreOption(vim, *args, **kwargs):
    '''VMFS datastore provisioning option that can be applied on a disk. VMFS
    datastores can be created or have their capacity increased using storage from a
    disk. There are often multiple ways in which extents can be allocated on a
    disk. Each instance of this structure represents one of the possible options
    that can be applied to provisiong VMFS datastore storage. Only options that
    follow ESX Server best practice guidelines will be presented.'''

    obj = vim.client.factory.create('{urn:vim25}VmfsDatastoreOption')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'info', 'spec' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def DatabaseSizeParam(vim, *args, **kwargs):
    '''DatabaseSizeParam contains information about a sample inventory. Using this
    information, database size requirements for that sample inventory can be
    computed. Depending on the accuracy of estimate desired, users can choose to
    specify the number of different types of managed entities. The numHosts and
    numVirtualMachines are the only two required fields. Rest are all optional
    fields filled up by Virtual Center based on some heuristics. These parameters
    need not represent a real inventory. The user can use these parameters to
    estimate the database size required by a hypothetical VirtualCenter setup.'''

    obj = vim.client.factory.create('{urn:vim25}DatabaseSizeParam')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'inventoryDesc' ]
    optional = [ 'perfStatsDesc', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def DailyTaskScheduler(vim, *args, **kwargs):
    '''The DailyTaskScheduler data object sets the time for daily task execution. You
    set the hour and the inherited minute property to complete the schedule. By
    default, the scheduled task will run once every day at the specified hour and
    minute.If you set the interval to a value greater than 1, the task will execute
    at the specified daily interval. (For example, an interval of 2 will cause the
    task to execute at the specified hour and minute every 2 days.)'''

    obj = vim.client.factory.create('{urn:vim25}DailyTaskScheduler')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'hour', 'minute', 'interval' ]
    optional = [ 'activeTime', 'expireTime', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineFlagInfoMonitorType = Enum(
    'debug',
    'release',
    'stats',

)



'''
The information here is taken from the `Perl Toolkit Programming Guide`_ found on the `VMware APIs and SDKs Documentation page`_.

You can provide the required parameters to pyvisdk utility applications in several different ways:

1. Pass parameters at the command line, using the required option flag followed by the value for the option:::

       --optionname optionvalue
   
2. Set as environment variables in a Linux profile; in the Environment properties dialog 
   of the Windows System control panel; or at the command-line, for the current session::
   
       set VI_SERVER=your_server_name

3. Create a configuration file (a plaintext file) that contains variable names (for the parameters) 
   and settings. Variables for the built-in options are shown in Table 1, below. Any option with the 
   variable attribute set in the python script or application (.py) can also be specified 
   in a configuration file.::

       VI_PROTOCOL=https
       VI_SERVER=prod01.vmware.com
       VI_SERVICEPATH=/sdk
       VI_USERNAME=Administrator
       VI_PASSWORD=*******

The pyvisdk runtime processes options set in the config file first, environment variables next, 
and command-line entries last. As shown in the example above, you can use a combination of these 
three approaches.

For example, assuming that most of your administrative chores take place through a central 
VirtualCenter Management Server, you can create a configuration file for executing applications 
and avoid repeatedly entering connection details. If you have multiple VirtualCenter Management 
Server or ESX Server systems and you need to administer them individually, you can create multiple 
configuration files using different names for each and then simply pass the --config option with 
the appropriate filename when you run the applications.

Furthermore, each option can be set in multiple ways. Since command-line flags take precedence over 
environment variables, and environment variables take precedence over settings in the configuration 
file, you can easily over-ride any parameter when necessary.
    
Table 1: Built-in Parameters and Variables

==============  ===============  =====================================================================================
Parameter       Variable         Description
==============  ===============  =====================================================================================
config          VI_CONFIG        Specify non-default name or location for the pyvisdk configuration file. 
                                 Default name and location for Linux is ~/.visdkrc and for Windows is %HOME%\visdk.rc.
help            ~                Displays usage information about a script or application to standard output 
                                 (command console).
password        VI_PASSWORD      for the specified username. Sucessful authentication with username and 
                                 password returns a session object that can be saved and used for subsequent 
                                 connections using the same or different script. See sessionfile.
portnumber      VI_PORTNUMBER    Port used for server connection.
protocol        VI_PROTOCOL      Protocol used to connect to server. Default is HTTPS. If the server has been 
                                 configured for HTTP, set to HTTP.
server          VI_SERVER        ESX Server or VirtualCenter Management Server host to which you want the 
                                 application or script to connect. Default is localhost if none specified.
servicepath     VI_SERVICEPATH   Service path for server connection. Default is /sdk/webService.
sessionfile     VI_SESSIONFILE   Name of file containing the token saved from successful login. Alternative 
                                 to specifying username and password. Sessions time out after 30 minutes 
                                 of inactivity.
url             VI_URL           Complete URL for SDK connection. An alternative to specifying protocol, 
                                 server, and servicepath as individual connection parameters. For example, 
                                 perl app_name.pl --url https://myserver.mycompany.com/sdk --username root 
                                 --password mypassword
username        VI_USERNAME      User account that has privileges to connect to the server.
verbose         VI_VERBOSE       Increase loglevel. Use in conjunction with Util::trace subroutine to display 
                                 additional debugging information. By default, value of --verbose (loglevel) 
                                 is 0.
version         ~                Displays script version information, if available.
==============  ===============  =====================================================================================


.. _Perl Toolkit Programming Guide: http://www.vmware.com/support/developer/viperltoolkit/doc/perl_toolkit_guide.html
.. _VMware APIs and SDKs Documentation page: http://www.vmware.com/support/pubs/sdk_pubs.html

'''
from optparse import OptionParser
from pyvisdk import Vim
import sys, logging
from options import Options

log = logging.getLogger(__name__)
log.setLevel(logging.INFO)

class PyvisdkApp(object):
    '''
    Base class implementation of a command line application.  See the :py:mod:`~pyvisdk.app` module for documentation on the format of the options file.
    '''


    def __init__(self, usage=None):
        '''
        Constructor
        '''
        self.options = Options()
        if not usage:
            usage = "usage: %prog [options]"
        self.parser = OptionParser(usage=usage)
        self.parser.add_option("-c", "--config", dest="VI_CONFIG", help="Specify non-default name or location for the VI Perl Toolkit configuration file. Default name and location for Linux is ~/.visdkrc and for Windows is %HOME%\visdk.rc.")
        self.parser.add_option("-p", "--password", dest="VI_PASSWORD", help="Password for the specified username. Sucessful authentication with username and password returns a session object that can be saved and used for subsequent connections using the same or different script. See sessionfile.")
        self.parser.add_option("--portnumber", dest="VI_PORTNUMBER", help="Port used for server connection.")
        self.parser.add_option("--protocol", dest="VI_PROTOCOL", help="Protocol used to connect to server. Default is HTTPS. If the server has been configured for HTTP, set to HTTP. ")
        self.parser.add_option("-s", "--server", dest="VI_SERVER", help="ESX Server or VirtualCenter Management Server host to which you want the application or script to connect. Default is localhost if none specified.")
        self.parser.add_option("--servicepath", dest="VI_SERVICEPATH", help="Service path for server connection. Default is /sdk/webService.")
        self.parser.add_option("--sessionfile", dest="VI_SESSIONFILE", help="Name of file containing the token saved from successful login. Alternative to specifying username and password. Sessions time out after 30 minutes of inactivity.")
        self.parser.add_option("--url", dest="VI_URL", help="Complete URL for SDK connection. An alternative to specifying protocol, server, and servicepath as individual connection parameters. For example, python app_name.py --url https://myserver.mycompany.com/sdk --username root --password mypassword")
        self.parser.add_option("-u", "--username", dest="VI_USERNAME", help="User account that has privileges to connect to the server.")
        self.parser.add_option("-v", "--verbose", dest="VI_VERBOSE", help="Increase loglevel. Use in conjunction with Util::trace subroutine to display additional debugging information. By default, value of --verbose (loglevel) is 0. ")
        self.parser.add_option("-V", "--version", help="Displays script version information, if available.")
        
    def parse(self):
        (cmd_opts, _) = self.parser.parse_args(sys.argv[1:]) # IGNORE W0201
        
        # load up options from the visdkrc file if there is any
        if cmd_opts.VI_CONFIG:
            self.options.load(cmd_opts.VI_CONFIG)
        else:
            self.options.load()
            
        # also, update options with environmental values, overriding those in the configuration file.
        self.options.load_env()
            
        # update our options with what was entered on the command line.  This will override previously
        # entered options
        for name, value in cmd_opts.__dict__.items():
            if value:
                self.options.update({name:value})
        
        # need to have server, username, and password or fail
        if not (self.options.VI_USERNAME and self.options.VI_PASSWORD and self.options.VI_SERVER):
            raise RuntimeError("Must specify --username, --password and --server")


    def connect(self):
        self.vim = Vim(self.options.VI_SERVER)
        self.vim.login(self.options.VI_USERNAME, self.options.VI_PASSWORD)
    
    def disconnect(self):
        self.vim.logout()



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualDiskAdapterType = Enum(
    'busLogic',
    'ide',
    'lsiLogic',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ProductComponentInfo(vim, *args, **kwargs):
    '''ProductComponentInfo data object type describes installed components. Product
    component is defined as a software module that is released and versioned
    independently but has dependency relationship with other products. If one
    product, for usability or any other reason, bundles other products,
    ProductComponentInfo type may be used to describe installed components. For
    example, ESX product may bundle VI Client in its releases.'''

    obj = vim.client.factory.create('{urn:vim25}ProductComponentInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'id', 'name', 'release', 'version' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class CustomizationSpecManager(BaseEntity):
    '''The CustomizationSpecManager managed object is used to manage customization
    specifications stored on the VirtualCenter server.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.CustomizationSpecManager):
        super(CustomizationSpecManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def encryptionKey(self):
        '''Gets a binary public encryption key that can be used to encrypt passwords in
        stored specifications.'''
        return self.update('encryptionKey')
    @property
    def info(self):
        '''Gets a list of information on available specifications.'''
        return self.update('info')

    
    
    def CheckCustomizationResources(self, guestOs):
        '''Validate that required resources are available on the server to customize a
        particular guest operating system. These would include sysprep for Windows and
        the debugfs and changefs volume editors for Linux guests.
        
        :param guestOs: Short name from the guest OS descriptor list describing the OS we intend to customize.
        
        '''
        return self.delegate("CheckCustomizationResources")(guestOs)
    
    def CreateCustomizationSpec(self, item):
        '''Creates a new specification.
        
        :param item: 
        
        '''
        return self.delegate("CreateCustomizationSpec")(item)
    
    def CustomizationSpecItemToXml(self, item):
        '''Converts a specification item to XML text
        
        :param item: 
        
        '''
        return self.delegate("CustomizationSpecItemToXml")(item)
    
    def DeleteCustomizationSpec(self, name):
        '''Deletes a specification.
        
        :param name: 
        
        '''
        return self.delegate("DeleteCustomizationSpec")(name)
    
    def DoesCustomizationSpecExist(self, name):
        '''Whether or not a specification exists.
        
        :param name: 
        
        '''
        return self.delegate("DoesCustomizationSpecExist")(name)
    
    def DuplicateCustomizationSpec(self, name, newName):
        '''Duplicates a specification.
        
        :param name: 
        
        :param newName: 
        
        '''
        return self.delegate("DuplicateCustomizationSpec")(name, newName)
    
    def GetCustomizationSpec(self, name):
        '''Obtains a specification for the given name.
        
        :param name: Unique name identifying the requested customization specification.
        
        '''
        return self.delegate("GetCustomizationSpec")(name)
    
    def OverwriteCustomizationSpec(self, item):
        '''Overwrites an existing specification, possibly after retrieving (by using
        'get') and editing it. If, based on the item's changeVersion value, the
        overwrite process detects that the specification has changed since its
        retrieval, then the API uses the SpecModified exception to warn the client that
        he might overwrite another client's change.
        
        :param item: 
        
        '''
        return self.delegate("OverwriteCustomizationSpec")(item)
    
    def RenameCustomizationSpec(self, name, newName):
        '''Renames a specification.
        
        :param name: 
        
        :param newName: 
        
        '''
        return self.delegate("RenameCustomizationSpec")(name, newName)
    
    def XmlToCustomizationSpecItem(self, specItemXml):
        '''Converts an XML string to a specification item
        
        :param specItemXml: 
        
        '''
        return self.delegate("XmlToCustomizationSpecItem")(specItemXml)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ProfileManager(BaseEntity):
    '''This Class is responsible for managing Profiles.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ProfileManager):
        super(ProfileManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def profile(self):
        '''A list of profiles known to this ProfileManager.'''
        return self.update('profile')

    
    
    def CreateProfile(self, createSpec):
        '''Create a profile from the specified CreateSpec.
        
        :param createSpec: Specification for the profile being created. Usually a derived class CreateSpec can be used to create the Profile.
        
        '''
        return self.delegate("CreateProfile")(createSpec)
    
    def FindAssociatedProfile(self, entity):
        '''Get the profile(s) to which this entity is associated. The list of profiles
        will only include profiles known to this profileManager.
        
        :param entity: Entity for which profile is being looked up.
        
        '''
        return self.delegate("FindAssociatedProfile")(entity)
    
    def QueryPolicyMetadata(self, policyName=None, profile=None):
        '''Get the Metadata information for the policyNames. PolicyNames are available
        with the defaultProfile obtained by invoking the method createDefaultProfile.
        
        :param policyName: Retrieve metadata for the specified policyNames. If policyName is not specified, metadata for all policies will be returned.
        
        :param profile: Base profile whose context needs to be used during the operationvSphere API 5.0
        
        '''
        return self.delegate("QueryPolicyMetadata")(policyName, profile)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.extensible_managed_object import ExtensibleManagedObject

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostVirtualNicManager(ExtensibleManagedObject):
    '''The VirtualNicManager managed object describes the special Virtual NIC
    configuration of the host.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostVirtualNicManager):
        super(HostVirtualNicManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def info(self):
        '''Network configuration.'''
        return self.update('info')

    
    
    def DeselectVnicForNicType(self, nicType, device):
        '''Deselect the VirtualNic to be a special type.
        
        :param nicType: The type of VirtualNic that would be deselected
        
        :param device: The device that uniquely identifies the VirtualNic.
        
        '''
        return self.delegate("DeselectVnicForNicType")(nicType, device)
    
    def QueryNetConfig(self, nicType):
        '''Get the NetConfig for the specified nicType
        
        :param nicType: The NicType
        
        '''
        return self.delegate("QueryNetConfig")(nicType)
    
    def SelectVnicForNicType(self, nicType, device):
        '''Select the NicType of the VirtualNic. Selecting a device automatically
        deselects the previous selection if NetConfig#multiSelectAllowed is false for
        the specified nicType. Else, the device is added to the list of selected nics.
        
        :param nicType: The type of VirtualNic that would be selected
        
        :param device: The device that uniquely identifies the VirtualNic.
        
        '''
        return self.delegate("SelectVnicForNicType")(nicType, device)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterRuleInfo(vim, *args, **kwargs):
    '''The ClusterRuleInfo data object is the base type for affinity and anti-affinity
    rules. The affinity and anti-affinity rules are DRS (Distributed Resource
    Scheduling) rules that affect the placement of virtual machines in a cluster.
    Hosts and virtual machines referenced in a DRS rule must be in the same
    cluster.Note: DRS rules are different than an individual host's CPU affinity
    rules (VirtualMachineAffinityInfo).The Server uses DRS rule objects to describe
    the current rule configuration (ClusterConfigInfoEx.rule). Your client
    application uses rule objects to configure the affinity and anti-affinity rules
    (ClusterConfigSpecEx.rulesSpec).You can create the following types of
    rules:Rule configuration is a dynamic process. When you create or modify a DRS
    rule, the Server applies the rule to the cluster. If the existing cluster
    configuration violates the rule, the Server attempts to correct the situation.
    If that is not possible, the Server generates a fault and produces a log event.
    DRS rules do not have precedence; all rules are applied equally. DRS does not
    validate one rule against another. If you create conflicting rules, the older
    rule takes precedence and DRS disables the newer rule.Improperly used, DRS
    rules can fragment the cluster and inhibit the proper functioning of DRS, HA,
    and DPM services. vSphere services never take any actions that would result in
    the violation of mandatory DRS rules. An operation that violates a mandatory
    rule would produce the following consequences.To avoid these situations,
    exercise caution when creating more than one mandatory rule, or consider using
    only optional rules. Make sure that the number of hosts with which a virtual
    machine is related by affinity rule is large enough that losing a host does not
    prevent the virtual machine from running.For manual and partially automated DRS
    clusters, the Server produces migration recommendations to satisfy the DRS
    rules. You are not required to act on the recommendations, but the Server
    maintains the recommendations until the rules are satisfied.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterRuleInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'enabled', 'inCompliance', 'key', 'mandatory', 'name', 'status', 'userCreated',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class IpPoolManager(BaseEntity):
    '''Singleton Managed Object used to manage IP Pools.IP Pools are used to allocate
    IPv4 and IPv6 addresses to vApps.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.IpPoolManager):
        super(IpPoolManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def CreateIpPool(self, dc, pool):
        '''Create a new IP pool.Create a new IP pool.Create a new IP pool.
        
        :param dc: The datacenter on which to create the pool.
        
        :param pool: The IP pool to create on the server
        
        '''
        return self.delegate("CreateIpPool")(dc, pool)
    
    def DestroyIpPool(self, dc, id, force):
        '''Destroys an IP pool on the given datacenter.Destroys an IP pool on the given
        datacenter.
        
        :param dc: The datacenter on which to find the pool
        
        :param id: The unique ID of the pool
        
        :param force: If true, the pool will be destroyed even if it is in use
        
        '''
        return self.delegate("DestroyIpPool")(dc, id, force)
    
    def QueryIpPools(self, dc):
        '''Return the list of IP pools for a datacenter.
        
        :param dc: The datacenter for which to look up the IP pools.
        
        '''
        return self.delegate("QueryIpPools")(dc)
    
    def UpdateIpPool(self, dc, pool):
        '''Update an IP pool on a datacenter.Update an IP pool on a datacenter.Update an
        IP pool on a datacenter.
        
        :param dc: The datacenter on which to look up the pool.
        
        :param pool: The IP pool to update on the server
        
        '''
        return self.delegate("UpdateIpPool")(dc, pool)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.profile import Profile

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostProfile(Profile):
    '''A host profile describes ESX Server configuration. The HostProfile managed
    object provides access to profile data and it defines methods to manipulate the
    profile. A host profile is a combination of subprofiles, each of which contains
    configuration data for a specific capability. Some examples of host
    capabilities are authentication, memory, networking, and security. For access
    to individual subprofiles, see the HostApplyProfile data object
    (HostProfile.config.applyProfile).Host profiles are part of the stateless
    configuration architecture. In the stateless environment, a Profile Engine runs
    on each ESX host, but an ESX host does not store its own configuration state.
    Instead, host configuration data is stored on vCenter Servers. Every time a
    host boots or reboots, it obtains its profile from the vCenter Server.* To
    create a base host profile use the HostProfileManager.CreateProfile method. To
    create a profile from an ESX host, specify a HostProfileHostBasedConfigSpec. To
    create a profile from a file, specify a HostProfileSerializedHostProfileSpec. *
    To create a subprofile for a particular host capability, use the
    HostProfileManager.CreateDefaultProfile method. After you create the default
    profile you can modify it and save it in the base profile. * To update an
    existing profile, use the HostProfile.UpdateHostProfile method. * To apply a
    host profile to an ESX host, use the ExecuteHostProfile method to generate
    configuration changes, then call the HostProfileManager.ApplyHostConfig_Task
    method to apply them.An individual host or a set of hosts may have some
    configuration settings that are different from the settings specified in the
    host profile. For example, the IP configuration for the host's virtual network
    adapters must be unique.* To verify host-specific data, use the parameter to
    the ExecuteHostProfile method. The Profile Engine will determine if you have
    specified all of the required parameters for the host configuration. If
    additional data is required, call the ExecuteHostProfile method again as many
    times as necessary to verify a complete set of parameters. * To apply host-
    specific data, use the parameter to the HostProfileManager.ApplyHostConfig_Task
    method.The Profile Engine saves host-specific data in an AnswerFile that is
    stored on the vCenter Server. The HostProfileManager provides several methods
    to manipulate answer files.You can create associations between hosts and
    profiles to support compliance checking. When you perform compliance checking,
    you can determine if a host configuration conforms to a host profile.* To
    create an association between a host and a profile, use the AssociateProfile
    method. The method adds the host to the HostProfile.entity[] list. * To
    retrieve the list of profiles associated with a host, use the
    HostProfileManager.FindAssociatedProfile method. * To check host compliance,
    use the CheckProfileCompliance_Task method. If you do not specify any hosts,
    the method will check the compliance of all hosts that are associated with the
    profile.You can also use the Profile Compliance Manager to check compliance by
    specifying profiles, entities (hosts), or both. See
    ProfileComplianceManager.CheckCompliance_Task.The vSphere architecture uses
    VMware profile plug-ins to define profile extensions. For information about
    using a plug-in to extend a host profile, see the VMware Technical Note .For
    access to host configuration data that is defined by plug-ins, use the
    ApplyProfile.policy[] and ApplyProfile.property[] lists. The HostApplyProfile
    and its subprofiles, which collectively define host configuration data, are
    derived from the ApplyProfile.* Policies store ESX configuration data in
    PolicyOption objects. * Profile property lists contain subprofiles defined by
    plug-ins. Subprofiles can be nested. * Subprofile lists are available as an
    extension to the base host profile
    (HostProfile.config.applyProfile.property[]). * Subprofile lists are available
    as extensions to the host subprofiles - for example, the network subprofile
    (HostApplyProfile.network.property[]).If you make changes to host profile data,
    later versions of profile plug-ins may not support the host configuration
    implied by the changes that you make. When a subsequent vSphere version becomes
    available, you must verify that the new version supports any previous
    configuration changes that you have made.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostProfile):
        super(HostProfile, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def referenceHost(self):
        '''Reference host in use for this host profile. To set this property, use the
        UpdateReferenceHost method. If you do not specify a host for validation
        (HostProfileCompleteConfigSpec.validatorHost), the Profile Engine uses the
        reference host to validate the profile.'''
        return self.update('referenceHost')

    
    
    def ExecuteHostProfile(self, host, deferredParam=None):
        '''Run the Profile Engine to determine the list of configuration changes needed
        for the specified host. The method generates configuration changes based on the
        host profile.Run the Profile Engine to determine the list of configuration
        changes needed for the specified host. The method generates configuration
        changes based on the host profile.
        
        :param host: Host on which to execute the profile. The host does not have to be associated with the profile.
        
        :param deferredParam: Additional configuration data to be applied to the host. This should contain all of the host-specific data, including data from from previous calls to the method.
        
        '''
        return self.delegate("ExecuteHostProfile")(host, deferredParam)
    
    def UpdateHostProfile(self, config):
        '''Update the <code>HostProfile</code> with the specified configuration data.
        
        :param config: Specification containing the new data.
        
        '''
        return self.delegate("UpdateHostProfile")(config)
    
    def UpdateReferenceHost(self, host=None):
        '''Sets the HostProfile.referenceHost property.
        
        :param host: Reference host to use. If unset, the referenceHost property is cleared.
        
        '''
        return self.delegate("UpdateReferenceHost")(host)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VAppAutoStartAction = Enum(
    'guestShutdown',
    'none',
    'powerOff',
    'powerOn',
    'suspend',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.compute_resource import ComputeResource

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ClusterComputeResource(ComputeResource):
    '''The ClusterComputeResource data object aggregates the compute resources of
    associated HostSystem objects into a single compute resource for use by virtual
    machines. The cluster services such as HA (High Availability), DRS (Distributed
    Resource Scheduling), and EVC (Enhanced vMotion Compatibility), enhance the
    utility of this single compute resource.Use the Folder.CreateClusterEx method
    to create an instance of this object.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ClusterComputeResource):
        super(ClusterComputeResource, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def actionHistory(self):
        '''The set of actions that have been performed recently.'''
        return self.update('actionHistory')
    @property
    def configuration(self):
        '''Configuration of the cluster.'''
        return self.update('configuration')
    @property
    def drsFault(self):
        '''A collection of the DRS faults generated in the last DRS invocation. Each
        element of the collection is the set of faults generated in one recommendation.
        DRS faults are generated when DRS tries to make recommendations for rule
        enforcement, power management, etc., and indexed in a tree structure with
        reason for recommendations and VM to migrate (optional) as the index keys.'''
        return self.update('drsFault')
    @property
    def drsRecommendation(self):
        '''If DRS is enabled, this returns the set of recommended migrations from the DRS
        module. The current set of recommendations may be empty, since there may be no
        recommended migrations at this time, or it is possible that DRS is not enabled.'''
        return self.update('drsRecommendation')
    @property
    def migrationHistory(self):
        '''The set of migration decisions that have recently been performed.'''
        return self.update('migrationHistory')
    @property
    def recommendation(self):
        '''List of recommended actions for the cluster. It is possible that the current
        set of recommendations may be empty, either due to not having any running
        dynamic recommendation generation module, or since there may be no recommended
        actions at this time.'''
        return self.update('recommendation')

    
    
    def AddHost_Task(self, spec, asConnected, resourcePool=None, license=None):
        '''Adds a host to the cluster. The hostname must be either an IP address, such as
        192.168.0.1, or a DNS resolvable name. DNS names may be fully qualified names,
        such as host1.domain1.com, or a short name such as host1, providing host1
        resolves to host1.domain1.com. The system uses DNS to resolve short names to
        fully qualified names. If the cluster supports nested resource pools and the
        user specifies the optional ResourcePool argument, then the host's root
        resource pool becomes the specified resource pool. The stand-alone host
        resource hierarchy is imported into the new nested resource pool.Adds a host to
        the cluster. The hostname must be either an IP address, such as 192.168.0.1, or
        a DNS resolvable name. DNS names may be fully qualified names, such as
        host1.domain1.com, or a short name such as host1, providing host1 resolves to
        host1.domain1.com. The system uses DNS to resolve short names to fully
        qualified names. If the cluster supports nested resource pools and the user
        specifies the optional ResourcePool argument, then the host's root resource
        pool becomes the specified resource pool. The stand-alone host resource
        hierarchy is imported into the new nested resource pool.
        
        :param spec: Specifies the host name, port, and password for the host to be added.
        
        :param asConnected: Flag to specify whether or not the host should be connected immediately after it is added. The host will not be added if a connection attempt is made and fails.
        
        :param resourcePool: the resource pool for the root resource pool from the host.
        
        :param license: Provide a licenseKey or licenseKeyType. See LicenseManagervSphere API 4.0
        
        '''
        return self.delegate("AddHost_Task")(spec, asConnected, resourcePool, license)
    
    def ApplyRecommendation(self, key):
        '''Applies a recommendation from the drsRecommendation or the recommendation list.
        Each recommendation can be applied only once.Applies a recommendation from the
        drsRecommendation or the recommendation list. Each recommendation can be
        applied only once.
        
        :param key: The key field of the DrsRecommendation or Recommendation.
        
        '''
        return self.delegate("ApplyRecommendation")(key)
    
    def CancelRecommendation(self, key):
        '''Cancels a recommendation. Currently only initial placement recommendations can
        be cancelled. Migration or power management recommendations cannot.
        
        :param key: The key field of the Recommendation.
        
        '''
        return self.delegate("CancelRecommendation")(key)
    
    def ClusterEnterMaintenanceMode(self, host, option=None):
        '''The API takes a list of hosts in the cluster as input, and returns a list of
        hosts in "ClusterMaintenanceResult" that the server can successfully evacuate
        given the existing constraints in the cluster, such as HA, FT, Vmotion
        compatibility, reservations, affinity rules, etc. The client is allowed to pass
        all hosts in the cluster to the API, even though all of them cannot enter
        maintenance mode at the same time. The list returned from the API contains the
        largest number of hosts that the server can evacuate simultaneously. The client
        can then request to enter each host in the returned list into maintenance mode.
        The client can specify an integer "DemandCapacityRatioTarget" option in the
        "option" parameter. The allowed values of the option range from 40 to 200, and
        the default value is 100. This option controls how much resource overcommitment
        the server should make in consolidating the VMs onto fewer hosts. A value of
        100 means the server will keep the same amount of powered-on capacity as the
        current VM demands. A value less than 100 means undercommitted resources. A
        value greater than 100 means overcommitted resources. The hosts are recommended
        based on the inventory at the time of the API invocation. It is not guaranteed
        that the actual enter-maintenance tasks on the hosts will succeed, if the
        inventory changes after the API returns, or if vmotions fail due to unexpected
        conditions. For possible exceptions thrown by the necessary relocate
        operations, see MigrateVM_Task.
        
        :param host: The array of hosts to put into maintenance mode.
        
        :param option: An array of OptionValue options for this query. The specified options override the advanced options in ClusterDrsConfigInfo.
        
        '''
        return self.delegate("ClusterEnterMaintenanceMode")(host, option)
    
    def MoveHostInto_Task(self, host, resourcePool=None):
        '''Moves an existing host into a cluster. The host must be part of the same
        datacenter, and if the host is part of a cluster, the host must be in
        maintenance mode.Moves an existing host into a cluster. The host must be part
        of the same datacenter, and if the host is part of a cluster, the host must be
        in maintenance mode.Moves an existing host into a cluster. The host must be
        part of the same datacenter, and if the host is part of a cluster, the host
        must be in maintenance mode.Moves an existing host into a cluster. The host
        must be part of the same datacenter, and if the host is part of a cluster, the
        host must be in maintenance mode.
        
        :param host: The list of hosts to move into the cluster.
        
        :param resourcePool: The resource pool to match the root resource pool of stand-alone hosts. This argument has no effect if the host is part of a cluster.
        
        '''
        return self.delegate("MoveHostInto_Task")(host, resourcePool)
    
    def MoveInto_Task(self, host):
        '''Moves an existing host into a cluster. The host must be part of the same
        datacenter, and if the host is part of a cluster, the host must be in
        maintenance mode.Moves an existing host into a cluster. The host must be part
        of the same datacenter, and if the host is part of a cluster, the host must be
        in maintenance mode.Moves an existing host into a cluster. The host must be
        part of the same datacenter, and if the host is part of a cluster, the host
        must be in maintenance mode.Moves an existing host into a cluster. The host
        must be part of the same datacenter, and if the host is part of a cluster, the
        host must be in maintenance mode.Moves an existing host into a cluster. The
        host must be part of the same datacenter, and if the host is part of a cluster,
        the host must be in maintenance mode.
        
        :param host: The list of hosts to move into the cluster.
        
        '''
        return self.delegate("MoveInto_Task")(host)
    
    def RecommendHostsForVm(self, vm, pool=None):
        '''<b>Deprecated.</b> <i>As of VI API 2.5, use PowerOnMultiVM_Task.
        RecommendHostsForVm cannot make any recommendations if DRS cannot find the
        specified host in the cluster. With PowerOnMultiVM_Task, DRS attempts to
        migrate virtual machines and power on hosts in standby mode, given the same
        conditions.</i>Gets a recommendation for where to power on, resume, revert from
        powered-off state to powered on state, or to migrate a specific virtual
        machine. If no host is found, an empty list is returned.
        
        :param vm: Specifies the virtual machine for which the user is requesting a recommendations.
        
        :param pool: Specifies the ResourcePool into which the virtual machine is to be migrated. If the virtual machine is powered-on, this argument must be specified and it is relevant only when the virtual machine is powered-on. This ResourcePool cannot be in the same cluster as the virtual machine.
        
        '''
        return self.delegate("RecommendHostsForVm")(vm, pool)
    
    def ReconfigureCluster_Task(self, spec, modify):
        '''<b>Deprecated.</b> <i>As of VI API 2.5, use
        ReconfigureComputeResource_Task.</i> Reconfigures a cluster.
        
        :param spec: A set of configuration changes to apply to the cluster. The specification can be a complete set of changes or a partial set of changes, applied incrementally.
        
        :param modify: Flag to specify whether the specification ("spec") should be applied incrementally. If "modify" is false and the operation succeeds, then the configuration of the cluster matches the specification exactly; in this case any unset portions of the specification will result in unset or default portions of the configuration.
        
        '''
        return self.delegate("ReconfigureCluster_Task")(spec, modify)
    
    def RefreshRecommendation(self):
        '''Make DRS invoke again and return a new list of recommendations. Concurrent
        "refresh" requests may be combined together and trigger only one DRS
        invocation.Make DRS invoke again and return a new list of recommendations.
        Concurrent "refresh" requests may be combined together and trigger only one DRS
        invocation.
        
        '''
        return self.delegate("RefreshRecommendation")()
    
    def RetrieveDasAdvancedRuntimeInfo(self):
        '''Retrieve DAS advanced runtime info for this cluster.
        
        '''
        return self.delegate("RetrieveDasAdvancedRuntimeInfo")()



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostScsiTopology(vim, *args, **kwargs):
    '''This data object type describes the SCSI topology information. The data objects
    in this data object type model the SCSI storage objects from a topological
    point of view. The SCSI topological view organizes objects by SCSI interface,
    which contain targets, which in turn contain logical units.SCSI Topology
    information is not guaranteed to exhaustively enumerate all storage devices on
    the system. It only shows storage devices that are actually enumerable form a
    host bus adapter. This means that only storage devices that are composed from
    one or more paths, which are in turn provided by a host bus adapter, will
    appear in this inventory.Storage devices provided by the native multipathing
    plugin (NMP) will always be represented in this inventory since NMP uses a
    simple policy to create devices out of the paths it claims.Examples of storage
    devices that will not appear in this inventory are logical devices that are not
    formed from directly claiming paths. Specific examples of devices that will not
    appear in this inventory include a device backed by a ramdisk or formed from a
    software RAID plugin.Legacy note: In hosts where HostPlugStoreTopology is not
    defined or does not exist on the HostStorageDeviceInfo object, only native
    multipathing exists. That means for these hosts, the ScsiTopology object
    contains the complete set of LUNs and targets available on the host.'''

    obj = vim.client.factory.create('{urn:vim25}HostScsiTopology')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'adapter', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ReplicationDiskConfigFaultReasonForFault = Enum(
    'diskNotFound',
    'diskTypeNotSupported',
    'duplicateDiskReplicationId',
    'invalidDiskKey',
    'invalidDiskReplicationId',
    'invalidPersistentFilePath',
    'reconfigureDiskReplicationIdNotAllowed',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

LinkDiscoveryProtocolConfigOperationType = Enum(
    'advertise',
    'both',
    'listen',
    'none',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineTicketType = Enum(
    'device',
    'guestControl',
    'mks',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualEthernetCardLegacyNetworkDeviceName = Enum(
    'bridged',
    'hostonly',
    'nat',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostCapabilityVmDirectPathGen2UnsupportedReason = Enum(
    'hostNptDisabled',
    'hostNptIncompatibleHardware',
    'hostNptIncompatibleProduct',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ResourceAllocationInfo(vim, *args, **kwargs):
    '''The ResourceAllocationInfo data object specifies the reserved resource
    requirement as well as the limit (maximum allowed usage) for a given kind of
    resource. This is specified for both memory allocation (specified in MB) and
    CPU allocation (specified in MHz).For a ResourcePool, the
    ResourceAllocationInfo object describes both the guaranteed amount of the
    resource (reservation) and whether or not it is expandable
    (expandableReservation). If expandableReservation is true, then the resource
    pool can grow its reservation dynamically by borrowing unreserved resources
    from its parent resource pool. For the methods CreateResourcePool, CreateVApp
    and ImportVApp, you must provide values for all properties except
    overheadLimit; they are not optional. (Currently, overheadLimit is for VMware
    internal use only.)If the limit is configured, it must be greater than or equal
    to the reservation.'''

    obj = vim.client.factory.create('{urn:vim25}ResourceAllocationInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'expandableReservation', 'limit', 'overheadLimit', 'reservation', 'shares',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ClusterDasConfigInfoHBDatastoreCandidate = Enum(
    'allFeasibleDs',
    'allFeasibleDsWithUserPreference',
    'userSelectedDs',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostCpuIdInfo(vim, *args, **kwargs):
    '''The CpuIdInfo data object type is used to represent the CPU features of a
    particular host or product, or to specify what the CPU feature requirements are
    for a particular virtual machine or guest operating system.For each register
    (eax,ebx,ecx,edx), the string is a bit mask of the form:When used to represent
    the features of a specific processor package (cpuPkg), the features common to
    all processors on a host (cpuFeature), or the features supported by a
    virtualization platform (supportedCpuFeature), each bit is either '0' or '1',
    or '-' for unknown/unspecified. In these feature vectors, the vendor field is
    never set.Optional values in these feature vectors default to
    '----:----:----:----:----:----:----:----'.When specifying the required feature
    set for a virtual machine or a guest operating system, the bits can take on the
    values as described below, and the vendor field may be set. The total feature
    requirements for a virtual machine are composed by using any requirements
    listed in the virtual machine's configuration to override the requirements
    listed in the descriptor for the virtual machine's guest OS.Bits used for
    specifying requirements:The values 'F' and '1' are rarely used but included for
    completeness. The '0' and '1' values do not promise a faithful virtualization
    of these features; whether the features work when forced to 0 or 1 is highly
    dependent on the guest software.Optional values in the requirements from the
    virtual machine's configuration default to
    '----:----:----:----:----:----:----:----'. Optional values in the requirements
    from the guest OS descriptor default to
    'xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx'.Once the feature requirements for a
    virtual machine have been composed from the virtual machine's configuration and
    guest OS descriptor, the bit types above are used to identify whether or not
    the virtual machine can be powered on or be migrated with VMotion to a
    particular host. The rules are as follows:'''

    obj = vim.client.factory.create('{urn:vim25}HostCpuIdInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'level' ]
    optional = [ 'eax', 'ebx', 'ecx', 'edx', 'vendor', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.extensible_managed_object import ExtensibleManagedObject

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostServiceSystem(ExtensibleManagedObject):
    '''The HostServiceSystem managed object describes the configuration of host
    services. This managed object operates in conjunction with the
    HostFirewallSystem managed object.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostServiceSystem):
        super(HostServiceSystem, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def serviceInfo(self):
        '''Service configuration.'''
        return self.update('serviceInfo')

    
    
    def RefreshServices(self):
        '''Refresh the service information and settings to pick up any changes made
        directly on the host.
        
        '''
        return self.delegate("RefreshServices")()
    
    def RestartService(self, id):
        '''Restarts the service.
        
        :param id: Service identifier (serviceInfo.service.key).
        
        '''
        return self.delegate("RestartService")(id)
    
    def StartService(self, id):
        '''Starts the service.
        
        :param id: Service identifier (serviceInfo.service.key).
        
        '''
        return self.delegate("StartService")(id)
    
    def StopService(self, id):
        '''Stops the service.
        
        :param id: Service identifier (serviceInfo.service.key).
        
        '''
        return self.delegate("StopService")(id)
    
    def UninstallService(self, id):
        '''Uninstalls the service. If the service is running, it is stopped before being
        uninstalled.
        
        :param id: Service identifier (serviceInfo.service.key).
        
        '''
        return self.delegate("UninstallService")(id)
    
    def UpdateServicePolicy(self, id, policy):
        '''Updates the activation policy of the service.
        
        :param id: Service identifier (serviceInfo.service.key).
        
        :param policy: Specifies the condition for service activation. Use one of the HostServicePolicy values.
        
        '''
        return self.delegate("UpdateServicePolicy")(id, policy)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ApplyStorageRecommendationResult(vim, *args, **kwargs):
    '''Both RecommendDatastores and DatastoreEnterMaintenanceMode methods may invoke
    Storage DRS for recommendations on placing or evacuating virtual disks. All
    initial placement recommendations, and some enterMaintenanceMode
    recommendations need to be approved by the user. Recommendations that are
    approved will be applied using the ApplyStorageDrsRecommendation_Task method.
    This class encapsulates the result of applying a subset of the
    recommendations.NOTE: This data object type and all of its methods are
    experimental and subject to change in future releases.'''

    obj = vim.client.factory.create('{urn:vim25}ApplyStorageRecommendationResult')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'vm', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DrsRecommendationReasonCode = Enum(
    'antiAffin',
    'fairnessCpuAvg',
    'fairnessMemAvg',
    'hostMaint',
    'jointAffin',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkIpInterfaceIpv6(Base):
    '''
    Commands to get and set IPv6 settings for vmknic.
    '''
    moid = 'ha-cli-handler-network-ip-interface-ipv6'
    def set(self, interfacename, enabledhcpv6=None, enablerouteradv=None, peerdns=None):
        '''
        Configure IPv6 settings for a given VMkernel network interface.
        :param enabledhcpv6: boolean, Setting this value to true will enable DHCPv6 on this interface and attempt to aquire an IPv6 address from the network 
        :param enablerouteradv: boolean, Setting this value to true will enable IPv6 Router Advertised IPv6 addresses to be added to this interface from any routers broadcasting on the local network.
        :param interfacename: string, The name of the VMkernel network interface to set IPv6 settings for. This name must be an interface listed in the interface list command.
        :param peerdns: boolean, A boolean value to indicate if the system should use the DNS settings published via DHCPv6 for this interface.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.ipv6.Set',
                            enabledhcpv6=enabledhcpv6,
                            enablerouteradv=enablerouteradv,
                            interfacename=interfacename,
                            peerdns=peerdns,
                            )
    def get(self, interfacename=None):
        '''
        Get IPv6 settings for VMkernel network interfaces. This does not include the IPv6 addresses which can be found in the listipv6 command
        :param interfacename: string, The name of the VMkernel network interface to limit the output of this command to.
        :returns: vim.EsxCLI.network.ip.interface.ipv6.get.IPv6Interface[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.ipv6.Get',
                            interfacename=interfacename,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class FcoeAdapter(Base):
    '''
    Operations that can be performed on FCOE-type storage HBAs
    '''
    moid = 'ha-cli-handler-fcoe-adapter'
    def list(self):
        '''
        List FCOE-capable CNA devices.
        :returns: vim.EsxCLI.fcoe.adapter.list.FcoeAdapterDevice[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.fcoe.adapter.List',
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostFirewallRulePortType = Enum(
    'dst',
    'src',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def UncommittedUndoableDisk(vim, *args, **kwargs):
    '''Fault thrown when an attempt is made to move or clone an undoable disk with an
    uncommitted REDO log. This is an error. Undoable disks may be moved but they
    must be committed.'''

    obj = vim.client.factory.create('{urn:vim25}UncommittedUndoableDisk')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj



'''
Created on Feb 15, 2011

@author: eplaster
'''
from suds.sudsobject import Property
import types, time
import logging
import urllib2
import lxml.etree as etree

from .base.base_entity import ManagedObjectReference, BaseEntity
from .base.data_object_types import DataObjectTypes
from .base.managed_object_types import ManagedObjectTypes
from .do.object_spec import ObjectSpec
from .do.traversal_spec import TraversalSpec
from .do.property_filter_spec import PropertyFilterSpec
from .do.property_spec import PropertySpec
from .do.selection_spec import SelectionSpec
from .do.retrieve_options import RetrieveOptions
from .enums.task_info_state import TaskInfoState
from .enums.sms_task_state import SmsTaskState
from .exceptions import VisdkTaskError
from .mo.service_instance import ServiceInstance
from .mo.task import Task
from .mo.sms_task import SmsTask
from .mo.sms_service_instance import SmsServiceInstance
from .utils import camel_to_under
from brownie.importing import import_string

from pyvisdk.transport import retry_on_URLError
log = logging.getLogger(__name__)
log.setLevel(logging.INFO)

HTTP_TUNNEL = 'sdkTunnel:8089'

class CoreMixin(object):

    def getObjectProperties(self, mobj, properties, parent=None):
        """
        * Retrieve contents for a single object based on the property collector
        * registered with the service.
        *
        * @param mobj Managed Object Reference to get contents for
        * @param properties names of properties of object to retrieve
        * @param mo_instance: Managed Object instance to update
        *
        * @return retrieved object contents
        """
        all = False
        if not properties:
            all = True

        pspec = PropertySpec(self, all=all, type=mobj._type, pathSet=properties)
        ospec = ObjectSpec(self, obj=ManagedObjectReference(mobj._type, mobj.value), skip=False)
        spec = PropertyFilterSpec(self, propSet=[pspec], objectSet=[ospec])

        obj_content = self.property_collector.RetrieveProperties([spec])
        return self._parse_object_content(obj_content, parent=parent)

    def getDecendentsByName(self, _type, properties=["name"], name=None, root=None):
        """
         * Get the ManagedObjectReference for an item under the specified root
         * folder that has the type and name specified.
         *
         * @param _type
         *            type of the managed object
         * @param properties
         *            names of properties of object to retrieve
         * @param name
         *            name to match
         * @param root
         *            a root folder if available, or null for default
         *
         * @return First ManagedObjectReference of the type / name pair found
        """
        if not "name" in properties:
            properties += ["name"]
        ocary = self.getContentsRecursively(_type=str(_type), props=properties, root=root)

        if not name:
            return ocary
        else:
            for obj in ocary:
                if obj.name == name:
                    return obj

    def getContentsRecursively(self, props=[], _type=None, collector=None, root=None, recurse=True):
        if not collector:
            collector = self.property_collector
        if not root:
            root = self.root
        if not _type:
            _type = ManagedObjectTypes.ManagedEntity #@UndefinedVariable

        pspec = PropertySpec(self, type=str(_type), pathSet=props)
        typeinfo = [ pspec ]

        selectionSpecs = []
        if recurse:
            selectionSpecs = self._buildFullTraversal()

        spec = PropertyFilterSpec(self, propSet=typeinfo, objectSet=[ ObjectSpec(self, root.ref, selectSet=selectionSpecs) ])
        obj_content = self.property_collector.RetrieveProperties([spec])
        return self._parse_object_content(obj_content)

    def _parse_object_content(self, obj_content, parent=None):
        rv = None
        class_name = obj_content.__class__.__name__
        class_name_upper = class_name[0].upper() + class_name[1:]
        if isinstance(obj_content, list):
            rv = []
            for obj in obj_content:
                rv.append(self._parse_object_content(obj, parent))

        elif type(obj_content) == types.NoneType:
            return obj_content

        elif issubclass(obj_content.__class__, BaseEntity):
            rv = obj_content

        elif class_name == 'ObjectContent':
            # create the managed object
            if not parent:
                rv = self._create_mo_obj(class_name, obj_content, parent)
            else:
                rv = []
                # now, run through the propSet
                if hasattr(obj_content, "propSet"):
                    for prop in obj_content.propSet:
                        rv.append(self._parse_object_content(prop.val))
                if len(rv) == 1:
                    rv = rv[0]

        elif class_name == "ManagedObjectReference":
            # managed object reference is too generic, so we get to the real type
            mo_f = import_string('pyvisdk.mo.%s.%s' % (camel_to_under(obj_content._type), obj_content._type))
            rv = mo_f(self, ref=obj_content)

        elif class_name in DataObjectTypes:
            rv = self._create_do_obj(class_name, obj_content, parent)

        elif class_name == 'Text':
            # obj_context is instance of suds.sax.text.Text which is a subclass of unicode
            # we need to return the text in proper encoding
            rv = str(obj_content.encode('UTF-8'))

        elif class_name in ['long', 'bool', 'int', 'datetime', 'str']:
            return obj_content

        elif "ArrayOf" in class_name:
            rv = []
            _type = class_name[7:]
            if not hasattr(obj_content, _type):
                _type = _type.lower()
            _list = getattr(obj_content, _type)
            for obj in _list:
                rv.append(self._parse_object_content(obj, parent))

        elif hasattr(obj_content, 'value') and hasattr(obj_content, '_type'):
            name = obj_content._type
            mod_name = 'pyvisdk.mo.%s' % camel_to_under(name)
            mo = import_string('%s.%s' % (mod_name, name))
            rv = mo(self, ref=obj_content)

        # TODO: ArrayOf....  ArrayOfManagedObjectReference ArrayOfAlarmState ArrayOfString ArrayOfInt ArrayOfPermission
        # TODO: val datetime datastore  entity host vm key

        elif class_name_upper in DataObjectTypes:
            rv = self._create_do_obj(class_name_upper, obj_content, parent)

        #elif class_name_upper in ManagedObjectTypes:
        #    rv = self._create_mo_obj(class_name_upper, obj_content, parent)

        else:
            log.warning("Unknown content type: %s" % class_name)
            rv = obj_content

        return rv

    def _create_do_obj(self, class_name, obj_content, parent):
        mod_name = 'pyvisdk.do.%s' % camel_to_under(class_name)
        do = import_string('%s.%s' % (mod_name, class_name))

        kwargs = {}
        for attr_name in filter(lambda x: not x.startswith('_'), dir(obj_content)):
            attr_data = eval('obj_content.%s' % attr_name)
            kwargs[attr_name] = self._parse_object_content(attr_data, parent=do)

        rv = do(self, **kwargs)
        return rv

    def _create_mo_obj(self, class_name, obj_content, parent):
        mo_f = import_string('pyvisdk.mo.%s.%s' % (camel_to_under(obj_content.obj._type), obj_content.obj._type))
        rv = mo_f(self, ref=obj_content.obj)

        # now, run through the propSet
        #if hasattr(obj_content, "propSet"):
        #    for prop in obj_content.propSet:
        #        # for each prop of the new object, create any data/managed objects
        #        setattr(rv, prop.name, self._parse_object_content(prop.val, parent=rv))
        return rv

    def _buildFullTraversal(self):
        """
         * This method creates a SelectionSpec[] to traverses the entire inventory
         * tree starting at a Folder
         *
         * @return The SelectionSpec[]
        """
        # Recurse through all ResourcePools
        rpToRp = TraversalSpec(self, name="rpToRp", type=ManagedObjectTypes.ResourcePool, path="resourcePool", #@UndefinedVariable
                        selectSet=[ SelectionSpec(self, name="rpToRp"), SelectionSpec(self, name="rpToVm") ])

        # Recurse through all ResourcePools
        rpToVm = TraversalSpec(self, name="rpToVm", type=ManagedObjectTypes.ResourcePool, path="vm") #@UndefinedVariable

        # Traversal through ResourcePool branch
        crToRp = TraversalSpec(self, name="crToRp", type=ManagedObjectTypes.ComputeResource, path="resourcePool", #@UndefinedVariable
                        selectSet=[ SelectionSpec(self, name="rpToRp"), SelectionSpec(self, name="rpToVm") ])

        # Traversal through host branch
        crToH = TraversalSpec(self, name="crToH", type=ManagedObjectTypes.ComputeResource, path="host") #@UndefinedVariable

        # Traversal through hostFolder branch
        dcToHf = TraversalSpec(self, name="dcToHf", type=ManagedObjectTypes.Datacenter, path="hostFolder", #@UndefinedVariable
                        selectSet=[SelectionSpec(self, name="visitFolders")])

        # Traversal through vmFolder branch
        dcToVmf = TraversalSpec(self, name="dcToVmf", type=ManagedObjectTypes.Datacenter, path="vmFolder", #@UndefinedVariable
                        selectSet=[SelectionSpec(self, name="visitFolders")])

        # Recurse through all Hosts
        HToVm = TraversalSpec(self, name="HToVm", type=ManagedObjectTypes.HostSystem, path="vm", #@UndefinedVariable
                        selectSet=[SelectionSpec(self, name="visitFolders")])

        # Recurse through the folders
        visitFolders = TraversalSpec(self, name="visitFolders", type=ManagedObjectTypes.Folder, path="childEntity", #@UndefinedVariable
                        selectSet=[ SelectionSpec(self, name="visitFolders"),
                                     SelectionSpec(self, name="dcToHf"),
                                     SelectionSpec(self, name="dcToVmf"),
                                     SelectionSpec(self, name="crToH"),
                                     SelectionSpec(self, name="crToRp"),
                                     SelectionSpec(self, name="HToVm"),
                                     SelectionSpec(self, name="rpToVm"), ])

        return (visitFolders, dcToVmf, dcToHf, crToH, crToRp, rpToRp, HToVm, rpToVm)

    def update(self, _type="", root=None, properties=[], version="", wait_time=2):
        if not root:
            root = self.root

        pSpec = PropertyFilterSpec(self,
                propSet=[PropertySpec(self, type=_type, pathSet=properties)],
                objectSet=[ObjectSpec(self, root.ref)])

        filter = self.property_collector.CreateFilter(pSpec, partialUpdates=False)

        changeData = self.property_collector.CheckForUpdates(version=version)

        # Destroy the filter when we are done.
        filter.DestroyPropertyFilter()

        if hasattr(changeData, "filterSet"):
            changeData = changeData.filterSet[0].objectSet[0]

        if hasattr(changeData, "changeSet"):
            changeData = changeData.changeSet

        return changeData, version


class VimBase(CoreMixin):
    '''
    Base class to hold the nuts and bolts for the web services grunt work.
    '''
    def __init__(self, server, connect=True, verbose=3, certfile=None, keyfile=None, wait_for_task=True):
        '''
        Constructor
        '''
        self.server_fqdn = self.server = server
        self.verbose = verbose
        self.connected = False
        self.certfile = certfile
        self.keyfile = keyfile
        self.proxy = None
        self.wait_for_task = wait_for_task
        if certfile and keyfile:
            self.server = HTTP_TUNNEL
            self.proxy = '%s:80' % server

        # setup logging...
        logging.getLogger('suds.client').setLevel(logging.INFO)
        logging.getLogger('suds.wsdl').setLevel(logging.INFO)
        logging.getLogger('suds.transport').setLevel(logging.INFO)
        logging.getLogger('suds.xsd.schema').setLevel(logging.INFO)

        if self.verbose > 5:
            logging.getLogger('suds.client').setLevel(logging.DEBUG)
            logging.getLogger('suds.transport').setLevel(logging.DEBUG)

        if connect:
            self.connect()

    def connect(self, timeout=1800):
        from .client import Client
        from .transport import HttpAuthenticated
        if self.connected:
            return

        self.client = Client(self.server, timeout=timeout)
        transport = HttpAuthenticated(proxy=self.proxy, certfile=self.certfile, keyfile=self.keyfile)
        self.client.client.set_options(transport=transport)
        # create the Service Instance managed object
        ref = Property('ServiceInstance')
        ref._type = 'ServiceInstance'
        self.service_instance = ServiceInstance(self, name='ServiceInstance', ref=ref)

        # get the service content
        self.service_content = self.service_instance.RetrieveServiceContent()
        self.property_collector = self.service_content.propertyCollector
        self.session_manager = self.service_content.sessionManager

        self.root = self.service_content.rootFolder
        self.connected = True

    def getVersions(self):
        versions = []

        def get_namespace_name(elem):
            for child in list(elem):
                if child.tag == "name":
                    return child.text

        url = urllib2.urlopen("https://" + self.server + "/sdk/vimServiceVersions.xml")
        root = etree.fromstring(url.read())
        names = root.findall(".//namespace")
        for namesp in names:
            if "urn:vim25" in get_namespace_name(namesp):
                versions = [x.text for x in namesp.findall(".//version")]
                log.debug("versions found: " + str(versions))
        return versions

    @retry_on_URLError
    def waitForTask(self, task, timeout=None):
        if not isinstance(task, Task):
            task = self._parse_object_content(task)

        log.debug("Waiting for task to complete...")
        t0 = time.time()
        while task.update("info.state") in [TaskInfoState.running, TaskInfoState.queued]:
            time.sleep(0.1)
            t1 = time.time()
            if timeout is not None and (t1 - t0) > timeout:
                raise VisdkTaskError("Timeout waiting for task")

        log.debug("Finished task...")

        if task.info.state == TaskInfoState.error:
            error = task.info.error
            raise VisdkTaskError(error.localizedMessage)

        return task.info.state


class SmsBase(CoreMixin):
    def __init__(self, vim, wait_for_task=True):
        super(SmsBase, self).__init__()
        self._vim = vim
        self.wait_for_task = wait_for_task

    def connect(self):
        from .client import SmsClient
        from suds.sax.element import Element
        self.client = SmsClient(self._vim.server_fqdn)
        session = eval(list(self._vim.client.client.options.transport.cookiejar)[0].value)
        cookie = Element("vcSessionCookie")
        cookie.setText(session)
        self.client.wsdl.options.__pts__.set("soapheaders", cookie)
        ref = Property('ServiceInstance')
        ref._type = 'ServiceInstance'
        self.service_instance = SmsServiceInstance(self, name='ServiceInstance', ref=ref)
        self.connected = True

    def waitForTask(self, task, timeout=None):
        if not isinstance(task, SmsTask):
            task = self._parse_object_content(task)

        log.debug("Waiting for task to complete...")
        t0 = time.time()
        while task.QuerySmsTaskInfo().state in [SmsTaskState.running, ]:
            time.sleep(0.1)
            t1 = time.time()
            if timeout is not None and (t1 - t0) > timeout:
                raise VisdkTaskError("Timeout waiting for task")

        log.debug("Finished task...")

        if task.QuerySmsTaskInfo().state == SmsTaskState.error:
            error = task.QuerySmsTaskInfo().error
            raise VisdkTaskError(error.localizedMessage)

        return task.QuerySmsTaskInfo().state




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualUSBUSBBackingInfo(vim, *args, **kwargs):
    '''The VirtualUSBUSBBackingInfo data object identifies a USB device on the host
    where the virtual machine is located. This type of backing supports only a
    local connection where the virtual machine will remain on the host to which the
    USB device is attached.To identify the USB device, you specify an autoconnect
    pattern for the deviceName. The virtual machine can connect to the USB device
    if the ESX server can find a USB device described by the autoconnect pattern.
    The autoconnect pattern consists of name:value pairs. You can use any
    combination of the following fields.For example, the following pattern
    identifies a USB device:This pattern identifies the USB device connected to
    port 1/3/0 on the host with the unique id .Special characters for autoconnect
    pattern values:'''

    obj = vim.client.factory.create('{urn:vim25}VirtualUSBUSBBackingInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'deviceName' ]
    optional = [ 'useAutoDetect', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineConfigInfoNpivWwnType = Enum(
    'external',
    'host',
    'vc',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def SnapshotCopyNotSupported(vim, *args, **kwargs):
    '''An attempt is being made to move or copy a virtual machine's disk that has
    associated snapshots, and preserving the snapshots is not supported because of
    some aspect of the virtual machine configuration, virtual machine power state,
    or the requested disk placement. This is an error for move operations (where
    the source is deleted after the copy) and a warning for clones (where the
    source is preserved).'''

    obj = vim.client.factory.create('{urn:vim25}SnapshotCopyNotSupported')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.managed_entity import ManagedEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class DistributedVirtualSwitch(ManagedEntity):
    '''The interface to the distributed virtual switch objects.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.DistributedVirtualSwitch):
        super(DistributedVirtualSwitch, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def capability(self):
        '''The capability of the switch. Queried against hosts, the property should always
        return configModificationSupported as false.'''
        return self.update('capability')
    @property
    def config(self):
        '''The configuration of the switch.'''
        return self.update('config')
    @property
    def networkResourcePool(self):
        '''The network resource pool information for the switch.'''
        return self.update('networkResourcePool')
    @property
    def portgroup(self):
        '''The portgroups that are defined on the switch.'''
        return self.update('portgroup')
    @property
    def summary(self):
        '''The summary of the switch.'''
        return self.update('summary')
    @property
    def uuid(self):
        '''The generated UUID of the switch. Unique across vCenter Server inventory and
        instances.'''
        return self.update('uuid')

    
    
    def AddDVPortgroup_Task(self, spec):
        '''Add a DistributedVirtualPortgroup to the switch.
        
        :param spec: The specification for the portgroup.
        
        '''
        return self.delegate("AddDVPortgroup_Task")(spec)
    
    def AddNetworkResourcePool(self, configSpec):
        '''Add a network resource pool.
        
        :param configSpec: the network resource pool configuration specification.
        
        '''
        return self.delegate("AddNetworkResourcePool")(configSpec)
    
    def EnableNetworkResourceManagement(self, enable):
        '''Enable/Disable network I/O control on the vSphere Distributed Switch.
        
        :param enable: If true, enables I/O control. If false, disables network I/O control.
        
        '''
        return self.delegate("EnableNetworkResourceManagement")(enable)
    
    def FetchDVPortKeys(self, criteria=None):
        '''Return the keys of ports that meet the criteria. Queried against host, the
        property reports only the connected ports currently on the host.
        
        :param criteria: The port selection criteria. If unset, the operation returns the keys of all the ports in the switch.
        
        '''
        return self.delegate("FetchDVPortKeys")(criteria)
    
    def FetchDVPorts(self, criteria=None):
        '''Return the ports that meet the criteria.
        
        :param criteria: The port selection criteria. If unset, the operation returns the keys of all the ports in the portgroup.
        
        '''
        return self.delegate("FetchDVPorts")(criteria)
    
    def MergeDvs_Task(self, dvs):
        '''Merge an existing DistributedVirtualSwitch (source) to this switch
        (destination). The host members and the connected entity of the source switch
        will be transferred to the destination switch. This operation disconnects the
        entities from the source switch, tears down its host proxy VirtualSwitches,
        creates new ones for the destination switch, and reconnects the entities to the
        destination switch.Merge an existing DistributedVirtualSwitch (source) to this
        switch (destination). The host members and the connected entity of the source
        switch will be transferred to the destination switch. This operation
        disconnects the entities from the source switch, tears down its host proxy
        VirtualSwitches, creates new ones for the destination switch, and reconnects
        the entities to the destination switch.Merge an existing
        DistributedVirtualSwitch (source) to this switch (destination). The host
        members and the connected entity of the source switch will be transferred to
        the destination switch. This operation disconnects the entities from the source
        switch, tears down its host proxy VirtualSwitches, creates new ones for the
        destination switch, and reconnects the entities to the destination switch.
        
        :param dvs: The switch (source) to be merged
        
        '''
        return self.delegate("MergeDvs_Task")(dvs)
    
    def MoveDVPort_Task(self, portKey, destinationPortgroupKey=None):
        '''Move the ports out of their current portgroup into the specified portgroup. If
        the moving of any of the ports results in a violation of the portgroup policy,
        or type of the source or destination portgroup, the operation raises a fault. A
        conflict port cannot be moved.
        
        :param portKey: The keys of the ports to be moved into the portgroup.
        
        :param destinationPortgroupKey: The key of the portgroup to be moved into. If unset, the port will be moved under the switch.
        
        '''
        return self.delegate("MoveDVPort_Task")(portKey, destinationPortgroupKey)
    
    def PerformDvsProductSpecOperation_Task(self, operation, productSpec=None):
        '''This method updates the DistributedVirtualSwitch product specifications.
        
        :param operation: The operation. See DistributedVirtualSwitchProductSpecOperationType for valid values. For VmwareDistributedVirtualSwitch, only upgrade is valid.
        
        :param productSpec: The product info of the implementation.
        
        '''
        return self.delegate("PerformDvsProductSpecOperation_Task")(operation, productSpec)
    
    def QueryUsedVlanIdInDvs(self):
        '''Return the used VLAN ID (PVLAN excluded) in the switch.
        
        '''
        return self.delegate("QueryUsedVlanIdInDvs")()
    
    def ReconfigureDVPort_Task(self, port):
        '''Reconfigure individual ports.
        
        :param port: The specification of the ports.
        
        '''
        return self.delegate("ReconfigureDVPort_Task")(port)
    
    def ReconfigureDvs_Task(self, spec):
        '''Reconfigure the switch.Reconfigure the switch.Reconfigure the switch.
        
        :param spec: The configuration of the switch
        
        '''
        return self.delegate("ReconfigureDvs_Task")(spec)
    
    def RectifyDvsHost_Task(self, hosts=None):
        '''<b>Deprecated.</b> <i>as of vSphere API 5.0. Use RectifyDvsOnHost_Task instead.
        Update the switch configuration on the host to bring them in sync with the
        current configuration in vCenter Server.</i> deprecated as of vSphere API 5.0.
        Use RectifyDvsOnHost_Task instead. Update the switch configuration on the host
        to bring them in sync with the current configuration in vCenter Server.
        
        :param hosts: The hosts to be rectified.
        
        '''
        return self.delegate("RectifyDvsHost_Task")(hosts)
    
    def RefreshDVPortState(self, portKeys=None):
        '''Refresh port states.
        
        :param portKeys: The keys of the ports to be refreshed. If not set, all port states are refreshed.
        
        '''
        return self.delegate("RefreshDVPortState")(portKeys)
    
    def RemoveNetworkResourcePool(self, key):
        '''Remove a network resource pool.
        
        :param key: the network resource pool key
        
        '''
        return self.delegate("RemoveNetworkResourcePool")(key)
    
    def UpdateDvsCapability(self, capability):
        '''Set the capability of the switch.
        
        :param capability: The capability of the switch.
        
        '''
        return self.delegate("UpdateDvsCapability")(capability)
    
    def UpdateNetworkResourcePool(self, configSpec):
        '''Update the network resource pool configuration.
        
        :param configSpec: the network resource pool configuration specification.
        
        '''
        return self.delegate("UpdateNetworkResourcePool")(configSpec)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostNumericSensorHealthState = Enum(
    'green',
    'red',
    'unknown',
    'yellow',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

NotSupportedDeviceForFTDeviceType = Enum(
    'paraVirtualSCSIController',
    'virtualVmxnet3',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterFailoverHostAdmissionControlPolicy(vim, *args, **kwargs):
    '''The ClusterFailoverHostAdmissionControlPolicy dedicates one or more hosts for
    use during failover. When a host fails with this policy in place, vSphere HA
    attempts to restart its virtual machines on a dedicated failover host. If this
    is not possible, for example the failover host itself has failed or it has
    insufficient resources, HA attempts to restart those virtual machines on
    another host in the cluster.To support the availabilty of a failover host, the
    vCenter Server will prevent users from powering on virtual machines on that
    host, or from using vMotion to migrate virtual machines to the host. Also, DRS
    does not use the failover host for load balancing.To obtain the status of a
    failover host, use the hostStatus property
    (ClusterComputeResourceSummary.admissionControlInfo.hostStatus).'''

    obj = vim.client.factory.create('{urn:vim25}ClusterFailoverHostAdmissionControlPolicy')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'failoverHosts', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

LicenseAssignmentFailedReason = Enum(
    'downgradeDisallowed',
    'hostsUnmanageableByVirtualCenterWithoutLicenseServer',
    'inventoryNotManageableByVirtualCenter',
    'keyEntityMismatch',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DiagnosticManagerLogFormat = Enum(
    'plain',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostDatastoreSystem(BaseEntity):
    '''This managed object creates and removes datastores from the host.To a host, a
    datastore is a storage abstraction that is backed by one of several types of
    storage volumes:An ESX Server system automatically discovers the VMFS volume on
    attached Logical Unit Numbers (LUNs) on startup and after re-scanning the host
    bus adapter. Datastores are automatically created. The datastore label is based
    on the VMFS volume label. If there is a conflict with an existing datastore, it
    is made unique by appending a suffix. The VMFS volume label will be
    unchanged.Destroying the datastore removes the partitions that compose the VMFS
    volume.Datastores are never automatically removed because transient storage
    connection outages may occur. They must be removed from the host using this
    interface.See Datastore'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostDatastoreSystem):
        super(HostDatastoreSystem, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def capabilities(self):
        '''Capability vector indicating the available product features.'''
        return self.update('capabilities')
    @property
    def datastore(self):
        '''List of datastores on this host.'''
        return self.update('datastore')

    
    
    def ConfigureDatastorePrincipal(self, userName, password=None):
        '''Configures datastore principal user for the host.Configures datastore principal
        user for the host.
        
        :param userName: Datastore principal user name.
        
        :param password: Optional password for systems that require password for user impersonation.
        
        '''
        return self.delegate("ConfigureDatastorePrincipal")(userName, password)
    
    def CreateLocalDatastore(self, name, path):
        '''Creates a new local datastore.
        
        :param name: The name of a datastore to create on the local host.
        
        :param path: The file path for a directory in which the virtual machine data will be stored.
        
        '''
        return self.delegate("CreateLocalDatastore")(name, path)
    
    def CreateNasDatastore(self, spec):
        '''Creates a new network-attached storage datastore.
        
        :param spec: The specification for creating a network-attached storage volume.
        
        '''
        return self.delegate("CreateNasDatastore")(spec)
    
    def CreateVmfsDatastore(self, spec):
        '''Creates a new VMFS datastore.
        
        :param spec: The specification for creating a datastore backed by a VMFS.
        
        '''
        return self.delegate("CreateVmfsDatastore")(spec)
    
    def ExpandVmfsDatastore(self, datastore, spec):
        '''Increases the capacity of an existing VMFS datastore by expanding (increasing
        the size of) an existing extent of the datastore.
        
        :param datastore: The datastore whose capacity should be increased.
        
        :param spec: The specification describing which extent of the VMFS datastore to expand.
        
        '''
        return self.delegate("ExpandVmfsDatastore")(datastore, spec)
    
    def ExtendVmfsDatastore(self, datastore, spec):
        '''Increases the capacity of an existing VMFS datastore by adding new extents to
        the datastore.
        
        :param datastore: The datastore whose capacity should be increased.
        
        :param spec: The specification describing what extents to add to a VMFS datastore.
        
        '''
        return self.delegate("ExtendVmfsDatastore")(datastore, spec)
    
    def QueryAvailableDisksForVmfs(self, datastore=None):
        '''Query to list disks that can be used to contain VMFS datastore extents. If the
        optional parameter name is supplied, queries for disks that can be used to
        contain extents for a VMFS datastore identified by the supplied name.
        Otherwise, the method retrieves disks that can be used to contain new VMFS
        datastores.Query to list disks that can be used to contain VMFS datastore
        extents. If the optional parameter name is supplied, queries for disks that can
        be used to contain extents for a VMFS datastore identified by the supplied
        name. Otherwise, the method retrieves disks that can be used to contain new
        VMFS datastores.Query to list disks that can be used to contain VMFS datastore
        extents. If the optional parameter name is supplied, queries for disks that can
        be used to contain extents for a VMFS datastore identified by the supplied
        name. Otherwise, the method retrieves disks that can be used to contain new
        VMFS datastores.
        
        :param datastore: The managed object reference of the VMFS datastore you want extents for.
        
        '''
        return self.delegate("QueryAvailableDisksForVmfs")(datastore)
    
    def QueryUnresolvedVmfsVolumes(self):
        '''Get the list of unbound VMFS volumes. For sharing a volume across hosts, a VMFS
        volume is bound to its underlying block device storage. When a low level block
        copy is performed to copy or move the VMFS volume, the copied volume will be
        unbound.
        
        '''
        return self.delegate("QueryUnresolvedVmfsVolumes")()
    
    def QueryVmfsDatastoreCreateOptions(self, devicePath, vmfsMajorVersion=None):
        '''Queries options for creating a new VMFS datastore for a disk.See devicePath
        
        :param devicePath: The devicePath of the disk on which datastore creation options are generated.See devicePath
        
        :param vmfsMajorVersion: major version of VMFS to be used for formatting the datastore. If this parameter is not specified, then the default VMFS version for the host is used.See devicePathvSphere API 5.0
        
        '''
        return self.delegate("QueryVmfsDatastoreCreateOptions")(devicePath, vmfsMajorVersion)
    
    def QueryVmfsDatastoreExpandOptions(self, datastore):
        '''Queries for options for increasing the capacity of an existing VMFS datastore
        by expanding (increasing the size of) an existing extent of the datastore.
        
        :param datastore: The datastore to be expanded.
        
        '''
        return self.delegate("QueryVmfsDatastoreExpandOptions")(datastore)
    
    def QueryVmfsDatastoreExtendOptions(self, datastore, devicePath, suppressExpandCandidates=None):
        '''Queries for options for increasing the capacity of an existing VMFS datastore
        by adding new extents using space from the specified disk.See devicePath
        
        :param datastore: The datastore to be extended.See devicePath
        
        :param devicePath: The devicePath of the disk on which datastore extension options are generated.See devicePath
        
        :param suppressExpandCandidates: Indicates whether to exclude options that can be used for extent expansion also. Free space can be used for adding an extent or expanding an existing extent. If this parameter is set to true, the list of options returned will not include free space that can be used for expansion.See devicePathvSphere API 4.0
        
        '''
        return self.delegate("QueryVmfsDatastoreExtendOptions")(datastore, devicePath, suppressExpandCandidates)
    
    def RemoveDatastore(self, datastore):
        '''Removes a datastore from a host.
        
        :param datastore: The datastore to be removed.
        
        '''
        return self.delegate("RemoveDatastore")(datastore)
    
    def ResignatureUnresolvedVmfsVolume_Task(self, resolutionSpec):
        '''Resignature an unbound VMFS volume. To safely enable sharing of the volume
        across hosts, a VMFS volume is bound to its underlying block device storage.
        When a low level block copy is performed to copy or move the VMFS volume, the
        copied volume will be unbound. In order for the VMFS volume to be usable, a
        resolution operation is needed to determine whether the VMFS volume should be
        treated as a new volume or not and what extents compose that volume in the
        event there is more than one unbound volume.Resignature an unbound VMFS volume.
        To safely enable sharing of the volume across hosts, a VMFS volume is bound to
        its underlying block device storage. When a low level block copy is performed
        to copy or move the VMFS volume, the copied volume will be unbound. In order
        for the VMFS volume to be usable, a resolution operation is needed to determine
        whether the VMFS volume should be treated as a new volume or not and what
        extents compose that volume in the event there is more than one unbound volume.
        
        :param resolutionSpec: A data object that describes what the disk extents to be used for creating the new VMFS volume.
        
        '''
        return self.delegate("ResignatureUnresolvedVmfsVolume_Task")(resolutionSpec)
    
    def UpdateLocalSwapDatastore(self, datastore=None):
        '''Choose the localSwapDatastore for this host. Any change to this setting will
        affect virtual machines that subsequently power on or resume from a suspended
        state at this host, or that migrate to this host while powered on; virtual
        machines that are currently powered on at this host will not yet be affected.
        
        :param datastore: The selected datastore. If this argument is unset, then the localSwapDatastore property becomes unset. Otherwise, the host must have read/write access to the indicated datastore.
        
        '''
        return self.delegate("UpdateLocalSwapDatastore")(datastore)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ServiceInstance(BaseEntity):
    '''The ServiceInstance managed object is the singleton root object of the
    inventory on both vCenter servers and servers running on standalone host
    agents. The server creates the ServiceInstance automatically, and also
    automatically creates the various manager entities that provide services in the
    virtual environment. Some examples of manager entities are LicenseManager,
    PerformanceManager, and ViewManager. You can access the manager entities
    through the content property.A vSphere API client application begins by
    connecting to a server and obtaining a reference to the ServiceInstance. The
    client can then use the RetrieveServiceContent method to gain access to the
    various vSphere manager entities and to the root folder of the inventory.When
    you create managed objects, the server adds them to the inventory. The
    inventory of managed objects includes instances the following object types:*
    ServiceInstance -- Root of the inventory; created by vSphere. * Datacenter -- A
    container that represents a virtual data center. It contains hosts, network
    entities, virtual machines and virtual applications, and datastores. * Folder
    -- A container used for hierarchical organization of the inventory. *
    VirtualMachine -- A virtual machine. * VirtualApp -- A virtual application. *
    ComputeResource -- A compute resource (either a cluster or a stand-alone host).
    * ResourcePool -- A subset of resources provided by a ComputeResource. *
    HostSystem -- A single host (ESX Server or VMware Server). * Network -- A
    network available to either hosts or virtual machines. *
    DistributedVirtualSwitch -- A distributed virtual switch. *
    DistributedVirtualPortgroup -- A distributed virtual port group. * Datastore --
    Platform-independent, host-independent storage for virtual machine files.The
    following figure shows the organization of managed objects in the vCenter
    hierarchy:Every Datacenter has the following set of dedicated folders. These
    folders are empty until you create entities for the Datacenter.* A folder for
    any combination of VirtualMachine and/or VirtualApp objects. VirtualApp objects
    can be nested, but only the parent VirtualApp can be visible in the folder.
    Virtual machines that are children of virtual applications are not associated
    with a VirtualMachine/VirtualApp folder. * A folder for a ComputeResource
    hierarchy. * A folder for network entities - any combination of Network,
    DistributedVirtualSwitch, and/or DistributedVirtualPortgroup objects. * A
    folder for Datastore objects.The host agent hierarchy has the same general form
    as the vCenter hierarchy, but most of the objects are limited to one instance:'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ServiceInstance):
        super(ServiceInstance, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def capability(self):
        '''API-wide capabilities.'''
        return self.update('capability')
    @property
    def content(self):
        '''The properties of the ServiceInstance managed object. The content property is
        identical to the return value from the RetrieveServiceContent method.'''
        return self.update('content')
    @property
    def serverClock(self):
        '''Contains the time most recently obtained from the server. The time is not
        necessarily current. This property is intended for use with the
        PropertyCollector WaitForUpdates method. The PropertyCollector will provide
        notification if some event occurs that changes the server clock time in a non-
        linear fashion.'''
        return self.update('serverClock')

    
    
    def CurrentTime(self):
        '''Returns the current time on the server. To monitor non-linear time changes, use
        the serverClock property.
        
        '''
        return self.delegate("CurrentTime")()
    
    def QueryVMotionCompatibility(self, vm, host, compatibility=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use
        QueryVMotionCompatibilityEx_Task instead.</i> Investigates the general VMotion
        compatibility of a virtual machine with a set of hosts. The virtual machine may
        be in any power state. Hosts may be in any connection state and also may be in
        maintenance mode.
        
        :param vm: The virtual machine that is the designated VMotion candidate.
        
        :param host: The group of hosts to analyze for compatibility.
        
        :param compatibility: The set of compatibility types to investigate. Each is a string chosen from VMotionCompatibilityType. If this argument is not set, then all compatibility types are investigated.
        
        '''
        return self.delegate("QueryVMotionCompatibility")(vm, host, compatibility)
    
    def RetrieveProductComponents(self):
        '''Component information for bundled products
        
        '''
        return self.delegate("RetrieveProductComponents")()
    
    def RetrieveServiceContent(self):
        '''Retrieves the properties of the service instance.
        
        '''
        return self.delegate("RetrieveServiceContent")()
    
    def ValidateMigration(self, vm, state=None, testType=None, pool=None, host=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use
        VirtualMachineProvisioningChecker instead.</i> Checks the validity of a set of
        proposed migrations. A migration is the act of changing the assigned execution
        host of a virtual machine, which can result from invoking MigrateVM_Task or
        RelocateVM_Task.
        
        :param vm: The set of virtual machines intended for migration.
        
        :param state: The power state that the virtual machines must have. If this argument is not set, each virtual machine is evaluated according to its current power state.
        
        :param testType: The set of tests to run. If this argument is not set, all tests will be run.
        
        :param pool: The target resource pool for the virtual machines. If the pool parameter is left unset, the target pool for each particular virtual machine's migration will be that virtual machine's current pool. If the virtual machine is a template then either this parameter or the host parameter must be set; additionally if resource tests are requested then this parameter is required.
        
        :param host: The target host on which the virtual machines will run. The host parameter may be left unset if the compute resource associated with the target pool represents a stand-alone host or a DRS-enabled cluster. In the former case the stand-alone host is used as the target host. In the latter case, each connected host in the cluster that is not in maintenance mode is tested as a target host. If the virtual machine is a template then either this parameter or the pool parameter must be set.
        
        '''
        return self.delegate("ValidateMigration")(vm, state, testType, pool, host)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemSecpolicyDomain(Base):
    '''
    Options related to VMkernel access control security domains.
    '''
    moid = 'ha-cli-handler-system-secpolicy-domain'
    def set(self, level, alldomains=None, name=None):
        '''
        Set the enforcement level for a domain in the system. Any option specified here is not persistent and will not survive a reboot of the system.
        :param alldomains: boolean, All domains.
        :param level: string, The enforcement level.
        :param name: string, The domain name.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.secpolicy.domain.Set',
                            alldomains=alldomains,
                            level=level,
                            name=name,
                            )
    def list(self):
        '''
        List the enforcement level for each domain.
        :returns: vim.EsxCLI.system.secpolicy.domain.list.DomainList[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.secpolicy.domain.List',
                            )   



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class GuestProcessManager(BaseEntity):
    '''ProcessManager is the managed object that provides APIs to manipulate the guest
    operating system processes.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.GuestProcessManager):
        super(GuestProcessManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def ListProcessesInGuest(self, vm, auth, pids=None):
        '''List the processes running in the guest operating system, plus those started by
        StartProgramInGuest that have recently completed.
        
        :param vm: Virtual machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param pids: If set, only return information about the specified processes. Otherwise, information about all processes are returned. If a specified processes does not exist, nothing will be returned for that process.
        
        '''
        return self.delegate("ListProcessesInGuest")(vm, auth, pids)
    
    def ReadEnvironmentVariableInGuest(self, vm, auth, names=None):
        '''Reads an environment variable from the guest OSReads an environment variable
        from the guest OS
        
        :param vm: Virtual machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param names: The names of the variables to be read. If not set, then all the environment variables are returned.
        
        '''
        return self.delegate("ReadEnvironmentVariableInGuest")(vm, auth, names)
    
    def StartProgramInGuest(self, vm, auth, spec):
        '''Starts a program in the guest operating system.Starts a program in the guest
        operating system.
        
        :param vm: Virtual machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param spec: The arguments describing the program to be started.
        
        '''
        return self.delegate("StartProgramInGuest")(vm, auth, spec)
    
    def TerminateProcessInGuest(self, vm, auth, pid):
        '''Terminates a process in the guest OS.
        
        :param vm: Virtual machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param pid: Process ID of the process to be terminated
        
        '''
        return self.delegate("TerminateProcessInGuest")(vm, auth, pid)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def OvfFileItem(vim, *args, **kwargs):
    '''An FileItem represents a file that must be uploaded by the caller when the
    inventory objects has been created in VI. These objects are created by
    ResourcePool.importVApp.Files can either be new files, in which case the
    "create" flag will be true, or updates to existing files in VI. The latter is
    used to support the OVF parentRef mechanism for Disks.'''

    obj = vim.client.factory.create('{urn:vim25}OvfFileItem')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'cimType', 'create', 'deviceId', 'path' ]
    optional = [ 'chunkSize', 'compressionMethod', 'size', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostSystemConnectionState = Enum(
    'connected',
    'disconnected',
    'notResponding',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostDatastoreBrowser(BaseEntity):
    '''The DatastoreBrowser managed object type provides access to the contents of one
    or more datastores. The items in a datastore are files that contain
    configuration, virtual disk, and the other data associated with a virtual
    machine.Although datastores may often be implemented using a traditional file
    system, a full interface to a file system is not provided here. Instead,
    specialized access for virtual machine files is provided. A datastore
    implementation may completely hide the file directory structure.The intent is
    to provide functionality analogous to a file chooser in a user interface.Files
    on datastores do not have independent permissions through this API. Instead,
    the permissions for all the files on a datastore derive from the datastore
    object itself. It is not possible to change individual file permissions as the
    user browsing the datastore may not necessarily be a recognized user from the
    point of view of the host changing the permission. This occurs if the user
    browsing the datastore is doing so through the VirtualCenter management
    server.The DatastoreBrowser provides many ways to customize a search for files.
    A search can be customized by specifying the types of files to be searched,
    search criteria specific to a file type, and the amount of detail about each
    file. The most basic queries only use file details and are efficient with
    limited side effects. For these queries, file metadata details can be
    optionally retrieved, but the files themselves are opened and their contents
    examined. As a result, these files are not necessarily validated.More
    complicated queries can be formed by specifying the specific types of files to
    be searched, the parameters to filter for each type, and the desired level of
    detail about each file. This method of searching for files is convenient
    because it allows additional data about the virtual machine component to be
    retrieved. In addition, certain validation checks can be performed on matched
    files as an inherent part of the details collection process. However, gathering
    extra details or the use of type specific filters can sometimes only be
    implemented by examining the contents of a file. As a result, the use of these
    conveniences comes with the cost of additional latency in the request and
    possible side effects on the system as a whole.The DatastoreBrowser is
    referenced from various objects, including from Datastore, ComputeResource,
    HostSystem and VirtualMachine. Depending on which object is used, there are
    different requirements for the accessibility of the browsed datastore from the
    host (or hosts) associated with the object:* When referenced from the target
    Datastore, it needs to be accessible from at least one host on which the
    datastore is mounted. See accessible. * When referenced from a ComputeResource,
    the target datastore needs to be accessible from at least one host in the
    ComputeResource. See accessible. * When referenced from a HostSystem, the
    target datastore needs to be accessible from that host. See accessible. * When
    referenced from a VirtualMachine, the target datastore needs to be accessible
    from the host on which the virtual machine is registered. See accessible.See
    FileInfo'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostDatastoreBrowser):
        super(HostDatastoreBrowser, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def datastore(self):
        '''Set of datastores that can be searched on this DatastoreBrowser.'''
        return self.update('datastore')
    @property
    def supportedType(self):
        '''The list of supported file types. The supported file types are represented as
        items in this list. For each supported file type, there is an object in the
        list whose dynamic type is one of the types derived from the FileQuery data
        object type. In general, the properties in this query type are not set.'''
        return self.update('supportedType')

    
    
    def DeleteFile(self, datastorePath):
        '''<b>Deprecated.</b> <i>As of VI API 2.5, use DeleteDatastoreFile_Task.</i>
        Deletes the specified files from the datastore. If a valid virtual disk file is
        specified, then all the components of the virtual disk are deleted.
        
        :param datastorePath: 
        
        '''
        return self.delegate("DeleteFile")(datastorePath)
    
    def SearchDatastore_Task(self, datastorePath, searchSpec=None):
        '''Returns the information for the files that match the given search criteria as a
        SearchResults object. Searches only the folder specified by the datastore path.
        The Datastore.Browse privilege must be held on the datastore identified by the
        datastore path.
        
        :param datastorePath: 
        
        :param searchSpec: 
        
        '''
        return self.delegate("SearchDatastore_Task")(datastorePath, searchSpec)
    
    def SearchDatastoreSubFolders_Task(self, datastorePath, searchSpec=None):
        '''Returns the information for the files that match the given search criteria as a
        SearchResults[] object. Searches the folder specified by the datastore path and
        all subfolders. The Datastore.Browse privilege must be held on the datastore
        identified by the datastore path.
        
        :param datastorePath: 
        
        :param searchSpec: 
        
        '''
        return self.delegate("SearchDatastoreSubFolders_Task")(datastorePath, searchSpec)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DasConfigFaultDasConfigFaultReason = Enum(
    'HostMisconfiguration',
    'HostNetworkMisconfiguration',
    'InsufficientPrivileges',
    'NoPrimaryAgentAvailable',
    'Other',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class SmsStorageManager(BaseEntity):
    '''The SmsStorageManager managed object (SMS) provides methods to retrieve
    information about available storage topology, capabilities, and state. SMS
    establishes and maintains connections with VASA providers. SMS retrieves
    information about storage availability from the providers, and clients can use
    the SMS API to perform the following operations.* Identify VASA providers. *
    Retrieve information about storage arrays. * Identify vSphere inventory
    entities (hosts and datastores) which are associated with external storage
    entities on the storage arrays.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.SmsStorageManager):
        super(SmsStorageManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def QueryArray(self, providerId):
        '''Get the list of storage arrays managed by all the registered VASA providers.
        
        :param providerId: List of uid for the VASA provider objects.
        
        '''
        return self.delegate("QueryArray")(providerId)
    
    def QueryArrayAssociatedWithLun(self, canonicalName):
        '''Get the StorageArray object that is associated with the ScsiLun.
        
        :param canonicalName: of ScsiLun
        
        '''
        return self.delegate("QueryArrayAssociatedWithLun")(canonicalName)
    
    def QueryDatastoreCapability(self, datastore):
        '''Get the capability for the given datastore.
        
        :param datastore: reference to
        
        '''
        return self.delegate("QueryDatastoreCapability")(datastore)
    
    def QueryDrsMigrationCapabilityForPerformance(self, srcDatastore, dstDatastore):
        '''<b>Deprecated.</b> <i>As of SMS API 3.0, use
        QueryDrsMigrationCapabilityForPerformanceEx</i> Query the provider to figure
        out whether Storage DRS should migrate VMDKs between the two given datastores.
        
        :param srcDatastore: Reference to the source
        
        :param dstDatastore: Reference to the destination
        
        '''
        return self.delegate("QueryDrsMigrationCapabilityForPerformance")(srcDatastore, dstDatastore)
    
    def QueryDrsMigrationCapabilityForPerformanceEx(self, datastore):
        '''Query available VASA providers for I/O performance based migration
        recommendations for all pair combinations of the given set of datastores.
        Datastore pairs for which a recommendation cannot be obtained are not included
        in the result.
        
        :param datastore: Array containing references toobjects.
        
        '''
        return self.delegate("QueryDrsMigrationCapabilityForPerformanceEx")(datastore)
    
    def QueryFileSystemAssociatedWithArray(self, arrayId):
        '''Get the StorageFileSystem data objects for the Array.
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryFileSystemAssociatedWithArray")(arrayId)
    
    def QueryHostAssociatedWithLun(self, scsi3Id, arrayId):
        '''Get HostSystem managed entities that share the StorageLun.
        
        :param scsi3Id: uuid for the StorageLun object.
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryHostAssociatedWithLun")(scsi3Id, arrayId)
    
    def QueryLunAssociatedWithArray(self, arrayId):
        '''Get the list of StorageLun data objects that for the Array.
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryLunAssociatedWithArray")(arrayId)
    
    def QueryLunAssociatedWithPort(self, portId, arrayId):
        '''Get the StorageLun data objects that are associated with StoragePort.
        
        :param portId: uuid for the StoragePort object.
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryLunAssociatedWithPort")(portId, arrayId)
    
    def QueryNfsDatastoreAssociatedWithFileSystem(self, fileSystemId, arrayId):
        '''Get NFS datastore managed entity that are associated with StorageFileSystem.
        
        :param fileSystemId: uuid for the StorageFileSystem object
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryNfsDatastoreAssociatedWithFileSystem")(fileSystemId, arrayId)
    
    def QueryPortAssociatedWithArray(self, arrayId):
        '''Get the StoragePort data objects that are associated with Array.
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryPortAssociatedWithArray")(arrayId)
    
    def QueryPortAssociatedWithLun(self, scsi3Id, arrayId):
        '''Get the StoragePort data object that is associated with LUN.
        
        :param scsi3Id: uuid for the StorageLun object.
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryPortAssociatedWithLun")(scsi3Id, arrayId)
    
    def QueryPortAssociatedWithProcessor(self, processorId, arrayId):
        '''Get the StoragePort data objects that are associated with Processor.
        
        :param processorId: uuid for the StorageProcessor object.
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryPortAssociatedWithProcessor")(processorId, arrayId)
    
    def QueryProcessorAssociatedWithArray(self, arrayId):
        '''Get the StorageProcessor data objects that are associated with Array.
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryProcessorAssociatedWithArray")(arrayId)
    
    def QueryProvider(self):
        '''Get the list of Providers that are currently registered with StorageManager.
        
        '''
        return self.delegate("QueryProvider")()
    
    def QueryVmfsDatastoreAssociatedWithLun(self, scsi3Id, arrayId):
        '''Get VMFS Datastore managed entity that are associated with StorageLun.
        
        :param scsi3Id: uuid for the StorageLun object
        
        :param arrayId: uuid for the StorageArray object.
        
        '''
        return self.delegate("QueryVmfsDatastoreAssociatedWithLun")(scsi3Id, arrayId)
    
    def RegisterProvider_Task(self, providerSpec):
        '''Register the provider and issue a sync operation on it.
        
        :param providerSpec: SmsProviderSpec containing parameters needed to register the provider
        
        '''
        return self.delegate("RegisterProvider_Task")(providerSpec)
    
    def UnregisterProvider_Task(self, providerId):
        '''Unregister the provider.
        
        :param providerId: uid for the provider
        
        '''
        return self.delegate("UnregisterProvider_Task")(providerId)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostDisconnectedEventReasonCode = Enum(
    'agentOutOfDate',
    'agentUpgrade',
    'insufficientLicenses',
    'licenseExpired',
    'passwordDecryptFailure',
    'sslThumbprintVerifyFailed',
    'unknown',
    'userRequest',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VmFaultToleranceInvalidFileBackingDeviceType = Enum(
    'virtualCdrom',
    'virtualDisk',
    'virtualFloppy',
    'virtualParallelPort',
    'virtualSerialPort',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ReadOnlyDisksWithLegacyDestination(vim, *args, **kwargs):
    '''The virtual machine uses read-only (undoable or nonpersistent) disks that can
    cause a slower power on at the migration destination. As a result, VMtion could
    slow down considerably or timeout. This is an issue only for migration of
    powered-on virtual machines from an ESX host with version greater than 2.0.x to
    an ESX host with version 2.0.x. It will be an error if the number of such disks
    is great enough to cause timeout ( >= 3 ), or a warning otherwise.'''

    obj = vim.client.factory.create('{urn:vim25}ReadOnlyDisksWithLegacyDestination')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 6:
        raise IndexError('Expected at least 7 arguments got: %d' % len(args))

    required = [ 'roDiskCount', 'timeoutDanger', 'dynamicProperty', 'dynamicType', 'faultCause',
        'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj
